# Copyright (C) Dnspython Contributors, see LICENSE for text of ISC license

import asyncio
import time

import pytest

import dns.asyncbackend
import dns.asyncresolver
import dns.resolver
import dns.nameserver

import tests.util


@pytest.mark.skipif(
    not tests.util.is_internet_reachable(), reason="Internet not reachable"
)
def test_basic_ddr_sync():
    for nameserver in ["1.1.1.1", "8.8.8.8"]:
        res = dns.resolver.Resolver(configure=False)
        res.nameservers = [nameserver]
        res.try_ddr()
        for nameserver in res.nameservers:
            assert isinstance(nameserver, dns.nameserver.Nameserver)
            assert nameserver.kind() != "Do53"


@pytest.mark.skipif(
    not tests.util.is_internet_reachable(), reason="Internet not reachable"
)
def test_basic_ddr_async():
    async def run():
        dns.asyncbackend._default_backend = None
        for nameserver in ["1.1.1.1", "8.8.8.8"]:
            res = dns.asyncresolver.Resolver(configure=False)
            res.nameservers = [nameserver]
            await res.try_ddr()
            for nameserver in res.nameservers:
                assert isinstance(nameserver, dns.nameserver.Nameserver)
                assert nameserver.kind() != "Do53"

    asyncio.run(run())



# Copyright (C) Dnspython Contributors, see LICENSE for text of ISC license

# Copyright (C) 2003-2007, 2009-2011 Nominum, Inc.
#
# Permission to use, copy, modify, and distribute this software and its
# documentation for any purpose with or without fee is hereby granted,
# provided that the above copyright notice and this permission notice
# appear in all copies.
#
# THE SOFTWARE IS PROVIDED "AS IS" AND NOMINUM DISCLAIMS ALL WARRANTIES
# WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
# MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL NOMINUM BE LIABLE FOR
# ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
# WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
# ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT
# OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.

import unittest

import dns.exception
import dns.tokenizer

Token = dns.tokenizer.Token


class TokenizerTestCase(unittest.TestCase):
    def testStr(self):
        tok = dns.tokenizer.Tokenizer("foo")
        token = tok.get()
        self.assertEqual(token, Token(dns.tokenizer.IDENTIFIER, "foo"))

    def testQuotedString1(self):
        tok = dns.tokenizer.Tokenizer(r'"foo"')
        token = tok.get()
        self.assertEqual(token, Token(dns.tokenizer.QUOTED_STRING, "foo"))

    def testQuotedString2(self):
        tok = dns.tokenizer.Tokenizer(r'""')
        token = tok.get()
        self.assertEqual(token, Token(dns.tokenizer.QUOTED_STRING, ""))

    def testQuotedString3(self):
        tok = dns.tokenizer.Tokenizer(r'"\"foo\""')
        token = tok.get()
        self.assertEqual(token, Token(dns.tokenizer.QUOTED_STRING, '\\"foo\\"'))

    def testQuotedString4(self):
        tok = dns.tokenizer.Tokenizer(r'"foo\010bar"')
        token = tok.get()
        self.assertEqual(token, Token(dns.tokenizer.QUOTED_STRING, "foo\\010bar"))

    def testQuotedString5(self):
        with self.assertRaises(dns.exception.UnexpectedEnd):
            tok = dns.tokenizer.Tokenizer(r'"foo')
            tok.get()

    def testQuotedString6(self):
        with self.assertRaises(dns.exception.SyntaxError):
            tok = dns.tokenizer.Tokenizer(r'"foo\01')
            tok.get()

    def testQuotedString7(self):
        with self.assertRaises(dns.exception.SyntaxError):
            tok = dns.tokenizer.Tokenizer('"foo\nbar"')
            tok.get()

    def testEmpty1(self):
        tok = dns.tokenizer.Tokenizer("")
        token = tok.get()
        self.assertTrue(token.is_eof())

    def testEmpty2(self):
        tok = dns.tokenizer.Tokenizer("")
        token1 = tok.get()
        token2 = tok.get()
        self.assertTrue(token1.is_eof() and token2.is_eof())

    def testEOL(self):
        tok = dns.tokenizer.Tokenizer("\n")
        token1 = tok.get()
        token2 = tok.get()
        self.assertTrue(token1.is_eol() and token2.is_eof())

    def testWS1(self):
        tok = dns.tokenizer.Tokenizer(" \n")
        token1 = tok.get()
        self.assertTrue(token1.is_eol())

    def testWS2(self):
        tok = dns.tokenizer.Tokenizer(" \n")
        token1 = tok.get(want_leading=True)
        self.assertTrue(token1.is_whitespace())

    def testComment1(self):
        tok = dns.tokenizer.Tokenizer(" ;foo\n")
        token1 = tok.get()
        self.assertTrue(token1.is_eol())

    def testComment2(self):
        tok = dns.tokenizer.Tokenizer(" ;foo\n")
        token1 = tok.get(want_comment=True)
        token2 = tok.get()
        self.assertEqual(token1, Token(dns.tokenizer.COMMENT, "foo"))
        self.assertTrue(token2.is_eol())

    def testComment3(self):
        tok = dns.tokenizer.Tokenizer(" ;foo bar\n")
        token1 = tok.get(want_comment=True)
        token2 = tok.get()
        self.assertEqual(token1, Token(dns.tokenizer.COMMENT, "foo bar"))
        self.assertTrue(token2.is_eol())

    def testMultiline1(self):
        tok = dns.tokenizer.Tokenizer("( foo\n\n bar\n)")
        tokens = list(iter(tok))
        self.assertEqual(
            tokens,
            [
                Token(dns.tokenizer.IDENTIFIER, "foo"),
                Token(dns.tokenizer.IDENTIFIER, "bar"),
            ],
        )

    def testMultiline2(self):
        tok = dns.tokenizer.Tokenizer("( foo\n\n bar\n)\n")
        tokens = list(iter(tok))
        self.assertEqual(
            tokens,
            [
                Token(dns.tokenizer.IDENTIFIER, "foo"),
                Token(dns.tokenizer.IDENTIFIER, "bar"),
                Token(dns.tokenizer.EOL, "\n"),
            ],
        )

    def testMultiline3(self):
        with self.assertRaises(dns.exception.SyntaxError):
            tok = dns.tokenizer.Tokenizer("foo)")
            list(iter(tok))

    def testMultiline4(self):
        with self.assertRaises(dns.exception.SyntaxError):
            tok = dns.tokenizer.Tokenizer("((foo)")
            list(iter(tok))

    def testUnget1(self):
        tok = dns.tokenizer.Tokenizer("foo")
        t1 = tok.get()
        tok.unget(t1)
        t2 = tok.get()
        self.assertEqual(t1, t2)
        self.assertEqual(t1.ttype, dns.tokenizer.IDENTIFIER)
        self.assertEqual(t1.value, "foo")

    def testUnget2(self):
        with self.assertRaises(dns.tokenizer.UngetBufferFull):
            tok = dns.tokenizer.Tokenizer("foo")
            t1 = tok.get()
            tok.unget(t1)
            tok.unget(t1)

    def testGetEOL1(self):
        tok = dns.tokenizer.Tokenizer("\n")
        t = tok.get_eol()
        self.assertEqual(t, "\n")

    def testGetEOL2(self):
        tok = dns.tokenizer.Tokenizer("")
        t = tok.get_eol()
        self.assertEqual(t, "")

    def testEscapedDelimiter1(self):
        tok = dns.tokenizer.Tokenizer(r"ch\ ld")
        t = tok.get()
        self.assertEqual(t.ttype, dns.tokenizer.IDENTIFIER)
        self.assertEqual(t.value, r"ch\ ld")

    def testEscapedDelimiter2(self):
        tok = dns.tokenizer.Tokenizer(r"ch\032ld")
        t = tok.get()
        self.assertEqual(t.ttype, dns.tokenizer.IDENTIFIER)
        self.assertEqual(t.value, r"ch\032ld")

    def testEscapedDelimiter3(self):
        tok = dns.tokenizer.Tokenizer(r"ch\ild")
        t = tok.get()
        self.assertEqual(t.ttype, dns.tokenizer.IDENTIFIER)
        self.assertEqual(t.value, r"ch\ild")

    def testEscapedDelimiter1u(self):
        tok = dns.tokenizer.Tokenizer(r"ch\ ld")
        t = tok.get().unescape()
        self.assertEqual(t.ttype, dns.tokenizer.IDENTIFIER)
        self.assertEqual(t.value, r"ch ld")

    def testEscapedDelimiter2u(self):
        tok = dns.tokenizer.Tokenizer(r"ch\032ld")
        t = tok.get().unescape()
        self.assertEqual(t.ttype, dns.tokenizer.IDENTIFIER)
        self.assertEqual(t.value, "ch ld")

    def testEscapedDelimiter3u(self):
        tok = dns.tokenizer.Tokenizer(r"ch\ild")
        t = tok.get().unescape()
        self.assertEqual(t.ttype, dns.tokenizer.IDENTIFIER)
        self.assertEqual(t.value, r"child")

    def testGetUInt(self):
        tok = dns.tokenizer.Tokenizer("1234")
        v = tok.get_int()
        self.assertEqual(v, 1234)
        with self.assertRaises(dns.exception.SyntaxError):
            tok = dns.tokenizer.Tokenizer('"1234"')
            tok.get_int()
        with self.assertRaises(dns.exception.SyntaxError):
            tok = dns.tokenizer.Tokenizer("q1234")
            tok.get_int()
        with self.assertRaises(dns.exception.SyntaxError):
            tok = dns.tokenizer.Tokenizer("281474976710656")
            tok.get_uint48()
        with self.assertRaises(dns.exception.SyntaxError):
            tok = dns.tokenizer.Tokenizer("4294967296")
            tok.get_uint32()
        with self.assertRaises(dns.exception.SyntaxError):
            tok = dns.tokenizer.Tokenizer("65536")
            tok.get_uint16()
        with self.assertRaises(dns.exception.SyntaxError):
            tok = dns.tokenizer.Tokenizer("256")
            tok.get_uint8()
        # Even though it is badly named get_int(), it's really get_unit!
        with self.assertRaises(dns.exception.SyntaxError):
            tok = dns.tokenizer.Tokenizer("-1234")
            tok.get_int()
        # get_uint16 can do other bases too, and has a custom error
        # for base 8.
        tok = dns.tokenizer.Tokenizer("177777")
        self.assertEqual(tok.get_uint16(base=8), 65535)
        with self.assertRaises(dns.exception.SyntaxError):
            tok = dns.tokenizer.Tokenizer("200000")
            tok.get_uint16(base=8)

    def testGetString(self):
        tok = dns.tokenizer.Tokenizer("foo")
        v = tok.get_string()
        self.assertEqual(v, "foo")
        tok = dns.tokenizer.Tokenizer('"foo"')
        v = tok.get_string()
        self.assertEqual(v, "foo")
        tok = dns.tokenizer.Tokenizer("abcdefghij")
        v = tok.get_string(max_length=10)
        self.assertEqual(v, "abcdefghij")
        with self.assertRaises(dns.exception.SyntaxError):
            tok = dns.tokenizer.Tokenizer("abcdefghij")
            tok.get_string(max_length=9)
        tok = dns.tokenizer.Tokenizer("")
        with self.assertRaises(dns.exception.SyntaxError):
            tok.get_string()

    def testMultiLineWithComment(self):
        tok = dns.tokenizer.Tokenizer("( ; abc\n)")
        tok.get_eol()
        # Nothing to assert here, as we're testing tok.get_eol() does NOT
        # raise.

    def testEOLAfterComment(self):
        tok = dns.tokenizer.Tokenizer("; abc\n")
        t = tok.get()
        self.assertTrue(t.is_eol())

    def testEOFAfterComment(self):
        tok = dns.tokenizer.Tokenizer("; abc")
        t = tok.get()
        self.assertTrue(t.is_eof())

    def testMultiLineWithEOFAfterComment(self):
        with self.assertRaises(dns.exception.SyntaxError):
            tok = dns.tokenizer.Tokenizer("( ; abc")
            tok.get_eol()

    def testEscapeUnexpectedEnd(self):
        with self.assertRaises(dns.exception.UnexpectedEnd):
            tok = dns.tokenizer.Tokenizer("\\")
            tok.get()

    def testEscapeBounds(self):
        with self.assertRaises(dns.exception.SyntaxError):
            tok = dns.tokenizer.Tokenizer("\\256")
            tok.get().unescape()
        with self.assertRaises(dns.exception.SyntaxError):
            tok = dns.tokenizer.Tokenizer("\\256")
            tok.get().unescape_to_bytes()

    def testGetUngetRegetComment(self):
        tok = dns.tokenizer.Tokenizer(";comment")
        t1 = tok.get(want_comment=True)
        tok.unget(t1)
        t2 = tok.get(want_comment=True)
        self.assertEqual(t1, t2)

    def testBadAsName(self):
        with self.assertRaises(dns.exception.SyntaxError):
            tok = dns.tokenizer.Tokenizer('"not an identifier"')
            t = tok.get()
            tok.as_name(t)

    def testBadGetTTL(self):
        with self.assertRaises(dns.exception.SyntaxError):
            tok = dns.tokenizer.Tokenizer('"not an identifier"')
            tok.get_ttl()

    def testBadGetEOL(self):
        with self.assertRaises(dns.exception.SyntaxError):
            tok = dns.tokenizer.Tokenizer('"not an identifier"')
            tok.get_eol_as_token()

    def testDanglingEscapes(self):
        for text in ['"\\"', '"\\0"', '"\\00"', '"\\00a"']:
            with self.assertRaises(dns.exception.SyntaxError):
                tok = dns.tokenizer.Tokenizer(text)
                tok.get().unescape()
            with self.assertRaises(dns.exception.SyntaxError):
                tok = dns.tokenizer.Tokenizer(text)
                tok.get().unescape_to_bytes()

    def testTokenMisc(self):
        t1 = dns.tokenizer.Token(dns.tokenizer.IDENTIFIER, "hi")
        t2 = dns.tokenizer.Token(dns.tokenizer.IDENTIFIER, "hi")
        t3 = dns.tokenizer.Token(dns.tokenizer.IDENTIFIER, "there")
        self.assertEqual(t1, t2)
        self.assertFalse(t1 == "hi")  # not NotEqual because we want to use ==
        self.assertNotEqual(t1, "hi")
        self.assertNotEqual(t1, t3)
        self.assertEqual(str(t1), '3 "hi"')

    def testBadConcatenateRemaining(self):
        with self.assertRaises(dns.exception.SyntaxError):
            tok = dns.tokenizer.Tokenizer('a b "not an identifier" c')
            tok.concatenate_remaining_identifiers()

    def testStdinFilename(self):
        tok = dns.tokenizer.Tokenizer()
        self.assertEqual(tok.filename, "<stdin>")

    def testBytesLiteral(self):
        tok = dns.tokenizer.Tokenizer(b"this is input")
        self.assertEqual(tok.get().value, "this")
        self.assertEqual(tok.filename, "<string>")
        tok = dns.tokenizer.Tokenizer(b"this is input", "myfilename")
        self.assertEqual(tok.filename, "myfilename")

    def testUngetBranches(self):
        tok = dns.tokenizer.Tokenizer(b"    this is input")
        t = tok.get(want_leading=True)
        tok.unget(t)
        t = tok.get(want_leading=True)
        self.assertEqual(t.ttype, dns.tokenizer.WHITESPACE)
        tok.unget(t)
        t = tok.get()
        self.assertEqual(t.ttype, dns.tokenizer.IDENTIFIER)
        self.assertEqual(t.value, "this")
        tok = dns.tokenizer.Tokenizer(b";    this is input\n")
        t = tok.get(want_comment=True)
        tok.unget(t)
        t = tok.get(want_comment=True)
        self.assertEqual(t.ttype, dns.tokenizer.COMMENT)
        tok.unget(t)
        t = tok.get()
        self.assertEqual(t.ttype, dns.tokenizer.EOL)


if __name__ == "__main__":
    unittest.main()



#!/usr/bin/env python3
#
# Small library and commandline tool to do logical diffs of zonefiles
# ./zonediff -h gives you help output
#
# Requires dnspython to do all the heavy lifting
#
# (c)2009 Dennis Kaarsemaker <dennis@kaarsemaker.net>
#
# Permission to use, copy, modify, and distribute this software and its
# documentation for any purpose with or without fee is hereby granted,
# provided that the above copyright notice and this permission notice
# appear in all copies.
#
# THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
# WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
# MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
# ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
# WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
# ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT
# OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
"""See diff_zones.__doc__ for more information"""

from typing import cast, Union, Any  # pylint: disable=unused-import

__all__ = ["diff_zones", "format_changes_plain", "format_changes_html"]

try:
    import dns.zone
    import dns.node
except ImportError:
    raise SystemExit("Please install dnspython")


def diff_zones(
    zone1,  # type: dns.zone.Zone
    zone2,  # type: dns.zone.Zone
    ignore_ttl=False,
    ignore_soa=False,
):  # type: (...) -> list
    """diff_zones(zone1, zone2, ignore_ttl=False, ignore_soa=False) -> changes
    Compares two dns.zone.Zone objects and returns a list of all changes
    in the format (name, oldnode, newnode).

    If ignore_ttl is true, a node will not be added to this list if the
    only change is its TTL.

    If ignore_soa is true, a node will not be added to this list if the
    only changes is a change in a SOA Rdata set.

    The returned nodes do include all Rdata sets, including unchanged ones.
    """

    changes = []
    for name in zone1:
        namestr = str(name)
        n1 = cast(dns.node.Node, zone1.get_node(namestr))
        n2 = cast(dns.node.Node, zone2.get_node(namestr))
        if not n2:
            changes.append((str(name), n1, n2))
        elif _nodes_differ(n1, n2, ignore_ttl, ignore_soa):
            changes.append((str(name), n1, n2))

    for name in zone2:
        n3 = cast(dns.node.Node, zone1.get_node(name))
        if not n3:
            n4 = cast(dns.node.Node, zone2.get_node(name))
            changes.append((str(name), n3, n4))
    return changes


def _nodes_differ(
    n1,  # type: dns.node.Node
    n2,  # type: dns.node.Node
    ignore_ttl,  # type: bool
    ignore_soa,  # type: bool
):  # type: (...) -> bool
    if ignore_soa or not ignore_ttl:
        # Compare datasets directly
        for r in n1.rdatasets:
            if ignore_soa and r.rdtype == dns.rdatatype.SOA:
                continue
            if r not in n2.rdatasets:
                return True
            if not ignore_ttl:
                return r.ttl != n2.find_rdataset(r.rdclass, r.rdtype).ttl

        for r in n2.rdatasets:
            if ignore_soa and r.rdtype == dns.rdatatype.SOA:
                continue
            if r not in n1.rdatasets:
                return True
        assert False
    else:
        return n1 != n2


def format_changes_plain(
    oldf,  # type: str
    newf,  # type: str
    changes,  # type: list
    ignore_ttl=False,
):  # type: (...) -> str
    """format_changes(oldfile, newfile, changes, ignore_ttl=False) -> str
    Given 2 filenames and a list of changes from diff_zones, produce diff-like
    output. If ignore_ttl is True, TTL-only changes are not displayed"""

    ret = "--- {}\n+++ {}\n".format(oldf, newf)
    for name, old, new in changes:
        ret += "@ %s\n" % name
        if not old:
            for r in new.rdatasets:
                ret += "+ %s\n" % str(r).replace("\n", "\n+ ")
        elif not new:
            for r in old.rdatasets:
                ret += "- %s\n" % str(r).replace("\n", "\n+ ")
        else:
            for r in old.rdatasets:
                if r not in new.rdatasets or (
                    r.ttl != new.find_rdataset(r.rdclass, r.rdtype).ttl
                    and not ignore_ttl
                ):
                    ret += "- %s\n" % str(r).replace("\n", "\n+ ")
            for r in new.rdatasets:
                if r not in old.rdatasets or (
                    r.ttl != old.find_rdataset(r.rdclass, r.rdtype).ttl
                    and not ignore_ttl
                ):
                    ret += "+ %s\n" % str(r).replace("\n", "\n+ ")
    return ret


def format_changes_html(
    oldf,  # type: str
    newf,  # type: str
    changes,  # type: list
    ignore_ttl=False,
):  # type: (...) -> str
    """format_changes(oldfile, newfile, changes, ignore_ttl=False) -> str
    Given 2 filenames and a list of changes from diff_zones, produce nice html
    output. If ignore_ttl is True, TTL-only changes are not displayed"""

    ret = """<table class="zonediff">
  <thead>
    <tr>
      <th>&nbsp;</th>
      <th class="old">%s</th>
      <th class="new">%s</th>
    </tr>
  </thead>
  <tbody>\n""" % (
        oldf,
        newf,
    )

    for name, old, new in changes:
        ret += '    <tr class="rdata">\n      <td class="rdname">%s</td>\n' % name
        if not old:
            for r in new.rdatasets:
                ret += (
                    '      <td class="old">&nbsp;</td>\n'
                    '      <td class="new">%s</td>\n'
                ) % str(r).replace("\n", "<br />")
        elif not new:
            for r in old.rdatasets:
                ret += (
                    '      <td class="old">%s</td>\n'
                    '      <td class="new">&nbsp;</td>\n'
                ) % str(r).replace("\n", "<br />")
        else:
            ret += '      <td class="old">'
            for r in old.rdatasets:
                if r not in new.rdatasets or (
                    r.ttl != new.find_rdataset(r.rdclass, r.rdtype).ttl
                    and not ignore_ttl
                ):
                    ret += str(r).replace("\n", "<br />")
            ret += "</td>\n"
            ret += '      <td class="new">'
            for r in new.rdatasets:
                if r not in old.rdatasets or (
                    r.ttl != old.find_rdataset(r.rdclass, r.rdtype).ttl
                    and not ignore_ttl
                ):
                    ret += str(r).replace("\n", "<br />")
            ret += "</td>\n"
        ret += "    </tr>\n"
    return ret + "  </tbody>\n</table>"


# Make this module usable as a script too.
def main():  # type: () -> None
    import argparse
    import subprocess
    import sys
    import traceback

    usage = """%prog zonefile1 zonefile2 - Show differences between zones in a diff-like format
%prog [--git|--bzr|--rcs] zonefile rev1 [rev2] - Show differences between two revisions of a zonefile

The differences shown will be logical differences, not textual differences.
"""
    p = argparse.ArgumentParser(usage=usage)
    p.add_argument(
        "-s",
        "--ignore-soa",
        action="store_true",
        default=False,
        dest="ignore_soa",
        help="Ignore SOA-only changes to records",
    )
    p.add_argument(
        "-t",
        "--ignore-ttl",
        action="store_true",
        default=False,
        dest="ignore_ttl",
        help="Ignore TTL-only changes to Rdata",
    )
    p.add_argument(
        "-T",
        "--traceback",
        action="store_true",
        default=False,
        dest="tracebacks",
        help="Show python tracebacks when errors occur",
    )
    p.add_argument(
        "-H",
        "--html",
        action="store_true",
        default=False,
        dest="html",
        help="Print HTML output",
    )
    p.add_argument(
        "-g",
        "--git",
        action="store_true",
        default=False,
        dest="use_git",
        help="Use git revisions instead of real files",
    )
    p.add_argument(
        "-b",
        "--bzr",
        action="store_true",
        default=False,
        dest="use_bzr",
        help="Use bzr revisions instead of real files",
    )
    p.add_argument(
        "-r",
        "--rcs",
        action="store_true",
        default=False,
        dest="use_rcs",
        help="Use rcs revisions instead of real files",
    )
    opts, args = p.parse_args()
    opts.use_vc = opts.use_git or opts.use_bzr or opts.use_rcs

    def _open(what, err):  # type: (Union[list,str], str) -> Any
        if isinstance(what, list):
            # Must be a list, open subprocess
            try:
                proc = subprocess.Popen(what, stdout=subprocess.PIPE)
                proc.wait()
                if proc.returncode == 0:
                    return proc.stdout
                sys.stderr.write(err + "\n")
            except Exception:
                sys.stderr.write(err + "\n")
                if opts.tracebacks:
                    traceback.print_exc()
        else:
            # Open as normal file
            try:
                return open(what, "rb")
            except IOError:
                sys.stderr.write(err + "\n")
                if opts.tracebacks:
                    traceback.print_exc()

    if not opts.use_vc and len(args) != 2:
        p.print_help()
        sys.exit(64)
    if opts.use_vc and len(args) not in (2, 3):
        p.print_help()
        sys.exit(64)

    # Open file descriptors
    if not opts.use_vc:
        oldn, newn = args
    else:
        if len(args) == 3:
            filename, oldr, newr = args
            oldn = "{}:{}".format(oldr, filename)
            newn = "{}:{}".format(newr, filename)
        else:
            filename, oldr = args
            newr = None
            oldn = "{}:{}".format(oldr, filename)
            newn = filename

    old, new = None, None
    oldz, newz = None, None
    if opts.use_bzr:
        old = _open(
            ["bzr", "cat", "-r" + oldr, filename],
            "Unable to retrieve revision {} of {}".format(oldr, filename),
        )
        if newr is not None:
            new = _open(
                ["bzr", "cat", "-r" + newr, filename],
                "Unable to retrieve revision {} of {}".format(newr, filename),
            )
    elif opts.use_git:
        old = _open(
            ["git", "show", oldn],
            "Unable to retrieve revision {} of {}".format(oldr, filename),
        )
        if newr is not None:
            new = _open(
                ["git", "show", newn],
                "Unable to retrieve revision {} of {}".format(newr, filename),
            )
    elif opts.use_rcs:
        old = _open(
            ["co", "-q", "-p", "-r" + oldr, filename],
            "Unable to retrieve revision {} of {}".format(oldr, filename),
        )
        if newr is not None:
            new = _open(
                ["co", "-q", "-p", "-r" + newr, filename],
                "Unable to retrieve revision {} of {}".format(newr, filename),
            )
    if not opts.use_vc:
        old = _open(oldn, "Unable to open %s" % oldn)
    if not opts.use_vc or newr is None:
        new = _open(newn, "Unable to open %s" % newn)

    if not old or not new:
        sys.exit(65)

    # Parse the zones
    try:
        oldz = dns.zone.from_file(old, origin=".", check_origin=False)
    except dns.exception.DNSException:
        sys.stderr.write("Incorrect zonefile: %s\n" % old)
        if opts.tracebacks:
            traceback.print_exc()
    try:
        newz = dns.zone.from_file(new, origin=".", check_origin=False)
    except dns.exception.DNSException:
        sys.stderr.write("Incorrect zonefile: %s\n" % new)
        if opts.tracebacks:
            traceback.print_exc()
    if not oldz or not newz:
        sys.exit(65)

    changes = diff_zones(oldz, newz, opts.ignore_ttl, opts.ignore_soa)
    changes.sort()

    if not changes:
        sys.exit(0)
    if opts.html:
        print(format_changes_html(oldn, newn, changes, opts.ignore_ttl))
    else:
        print(format_changes_plain(oldn, newn, changes, opts.ignore_ttl))
    sys.exit(1)


if __name__ == "__main__":
    main()



import sys

import trio

import dns.message
import dns.asyncquery
import dns.asyncresolver


async def main():
    if len(sys.argv) > 1:
        host = sys.argv[0]
    else:
        host = "www.dnspython.org"
    q = dns.message.make_query(host, "A")
    r = await dns.asyncquery.udp(q, "8.8.8.8")
    print(r)
    q = dns.message.make_query(host, "A")
    r = await dns.asyncquery.tcp(q, "8.8.8.8")
    print(r)
    q = dns.message.make_query(host, "A")
    r = await dns.asyncquery.tls(q, "8.8.8.8")
    print(r)
    a = await dns.asyncresolver.resolve(host, "A")
    print(a.response)
    zn = await dns.asyncresolver.zone_for_name(host)
    print(zn)
    answer = await dns.asyncresolver.resolve_at("8.8.8.8", "amazon.com", "NS")
    print("The amazon.com nameservers are:")
    for rr in answer:
        print(rr.target)


if __name__ == "__main__":
    trio.run(main)



#!/usr/bin/python
"""
Minimalistic RFC 1996-like NOTIFY sender.
"""
import argparse
import ipaddress
import socket

import dns.flags
import dns.inet
import dns.message
import dns.name
import dns.opcode
import dns.query
import dns.rrset
import dns.rdataclass
import dns.rdatatype


def main():
    """Also prints all inputs and intermediate values"""
    parser = argparse.ArgumentParser(
        description="Send DNS NOTIFY mesage via UDP, optionally with synthetized SOA RR "
        "in ANSWER section. No checks. It's not RFC 1996 sect 3.6 compliant sender."
    )
    parser.add_argument("--source", type=ipaddress.ip_address, help="source IP address")
    parser.add_argument("--port", type=int, help="target port", default=53)
    parser.add_argument("target", type=ipaddress.ip_address, help="target IP address")
    parser.add_argument("zone", type=dns.name.from_text)
    parser.add_argument(
        "serial",
        nargs="?",
        type=int,
        help="optional serial - adds SOA RR into ANSWER section",
    )
    parser.add_argument(
        "--rdclass",
        default=dns.rdataclass.IN,
        type=dns.rdataclass.from_text,
        help="DNS class, defaults to IN",
    )
    args = parser.parse_args()
    if args.source:
        if dns.inet.af_for_address(str(args.target)) != dns.inet.af_for_address(
            str(args.source)
        ):
            parser.error("address family for source and target must be the same")

    print(args)
    msg = construct_msg(args)
    print(msg)
    udp_send(msg, args)


def construct_msg(args):
    """if args.serial is specified it creates fake SOA RR with given serial"""
    msg = dns.message.make_query(
        args.zone, dns.rdatatype.SOA, rdclass=args.rdclass, flags=dns.flags.AA
    )
    msg.set_opcode(dns.opcode.NOTIFY)
    if args.serial:
        soa = dns.rrset.from_text_list(
            name=args.zone,
            ttl=0,
            rdclass=args.rdclass,
            rdtype=dns.rdatatype.SOA,
            text_rdatas=[f". . {args.serial} 0 0 0 0"],
        )
        msg.answer.append(soa)
    return msg


def udp_send(msg, args):
    """ignores checks prescribed by RFC 1996 sect 3.6"""
    afam = dns.inet.af_for_address(str(args.target))
    sock = socket.socket(afam, socket.SOCK_DGRAM)
    if args.source:
        sock.bind((str(args.source), 0))
    dns.query.send_udp(sock, what=msg, destination=(str(args.target), args.port))


if __name__ == "__main__":
    main()



# Copyright (C) 2014 Red Hat, Inc.
# Author: Petr Spacek <pspacek@redhat.com>
#
# Permission to use, copy, modify, and distribute this software and its
# documentation for any purpose with or without fee is hereby granted,
# provided that the above copyright notice and this permission notice
# appear in all copies.
#
# THE SOFTWARE IS PROVIDED 'AS IS' AND RED HAT DISCLAIMS ALL WARRANTIES
# WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
# MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL NOMINUM BE LIABLE FOR
# ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
# WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
# ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT
# OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.

import unittest

import dns.rrset
import dns.rdtypes.ANY.DNSKEY

from typing import Set  # pylint: disable=unused-import


class RdtypeAnyDnskeyTestCase(unittest.TestCase):
    def testFlagsAll(self):  # type: () -> None
        """Test that all defined flags are recognized."""
        good_s = {"SEP", "REVOKE", "ZONE"}
        good_f = 0x181
        self.assertEqual(
            dns.rdtypes.ANY.DNSKEY.SEP
            | dns.rdtypes.ANY.DNSKEY.REVOKE
            | dns.rdtypes.ANY.DNSKEY.ZONE,
            good_f,
        )

    def testFlagsRRToText(self):  # type: () -> None
        """Test that RR method returns correct flags."""
        rr = dns.rrset.from_text("foo", 300, "IN", "DNSKEY", "257 3 8 KEY=")[0]
        self.assertEqual(
            dns.rdtypes.ANY.DNSKEY.ZONE | dns.rdtypes.ANY.DNSKEY.SEP, rr.flags
        )


if __name__ == "__main__":
    unittest.main()



import dns.rdatatype

print("Rdatatypes")
print("----------")
print()
by_name = {}
for rdtype in dns.rdatatype.RdataType:
    short_name = dns.rdatatype.to_text(rdtype).replace("-", "_")
    by_name[short_name] = int(rdtype)
for k in sorted(by_name.keys()):
    v = by_name[k]
    print(f".. py:data:: dns.rdatatype.{k}")
    print(f"   :annotation: = {v}")



# Copyright (C) Dnspython Contributors, see LICENSE for text of ISC license

import socket
import sys
import unittest

import dns.name
import dns.rdataclass
import dns.rdatatype
import dns.resolver
import tests.util


@unittest.skipIf(not tests.util.is_internet_reachable(), "Internet not reachable")
class OverrideSystemResolverTestCase(unittest.TestCase):
    def setUp(self):
        self.res = dns.resolver.Resolver(configure=False)
        self.res.nameservers = ["8.8.8.8"]
        self.res.cache = dns.resolver.LRUCache()
        dns.resolver.override_system_resolver(self.res)

    def tearDown(self):
        dns.resolver.restore_system_resolver()
        self.res = None

    def test_override(self):
        self.assertTrue(socket.getaddrinfo is dns.resolver._getaddrinfo)
        socket.gethostbyname("www.dnspython.org")
        answer = self.res.cache.get(
            (
                dns.name.from_text("www.dnspython.org."),
                dns.rdatatype.A,
                dns.rdataclass.IN,
            )
        )
        self.assertTrue(answer is not None)
        self.res.cache.flush()
        socket.gethostbyname_ex("www.dnspython.org")
        answer = self.res.cache.get(
            (
                dns.name.from_text("www.dnspython.org."),
                dns.rdatatype.A,
                dns.rdataclass.IN,
            )
        )
        self.assertTrue(answer is not None)
        self.res.cache.flush()
        socket.getfqdn("8.8.8.8")
        answer = self.res.cache.get(
            (
                dns.name.from_text("8.8.8.8.in-addr.arpa."),
                dns.rdatatype.PTR,
                dns.rdataclass.IN,
            )
        )
        self.assertTrue(answer is not None)
        self.res.cache.flush()
        socket.gethostbyaddr("8.8.8.8")
        answer = self.res.cache.get(
            (
                dns.name.from_text("8.8.8.8.in-addr.arpa."),
                dns.rdatatype.PTR,
                dns.rdataclass.IN,
            )
        )
        self.assertTrue(answer is not None)
        # restoring twice is harmless, so we restore now instead of
        # waiting for tearDown so we can assert that it worked
        dns.resolver.restore_system_resolver()
        self.assertTrue(socket.getaddrinfo is dns.resolver._original_getaddrinfo)

    def equivalent_info(self, a, b, q):
        if len(a) != len(b):
            return False
        for x in a:
            if x not in b:
                # Windows does not set the protocol to non-zero, so try
                # looking for a zero protocol.
                y = (x[0], x[1], 0, x[3], x[4])
                if y not in b:
                    # musl libc insists on always providing a canonical name, so
                    # accept that too.
                    y = (x[0], x[1], x[2], q, x[4])
                    if y not in b:
                        print("NOT EQUIVALENT")
                        print(a)
                        print(b)
                        return False
        return True

    def equivalent(self, *args, **kwargs):
        q = args[0]
        a = socket.getaddrinfo(*args, **kwargs)
        b = dns.resolver._original_getaddrinfo(*args, **kwargs)
        return self.equivalent_info(a, b, q)

    @unittest.skipIf(
        sys.platform == "win32", "avoid windows original getaddrinfo issues"
    )
    def test_basic_getaddrinfo(self):
        self.assertTrue(
            self.equivalent("dns.google", 53, socket.AF_INET, socket.SOCK_DGRAM)
        )
        self.assertTrue(
            self.equivalent("dns.google", 53, socket.AF_INET6, socket.SOCK_DGRAM)
        )
        self.assertTrue(
            self.equivalent("dns.google", None, socket.AF_UNSPEC, socket.SOCK_DGRAM)
        )
        self.assertTrue(
            self.equivalent("8.8.8.8", 53, socket.AF_INET, socket.SOCK_DGRAM)
        )
        self.assertTrue(
            self.equivalent(
                "2001:4860:4860::8888", 53, socket.AF_INET6, socket.SOCK_DGRAM
            )
        )
        self.assertTrue(
            self.equivalent(
                "8.8.8.8",
                53,
                socket.AF_INET,
                socket.SOCK_DGRAM,
                flags=socket.AI_NUMERICHOST,
            )
        )
        self.assertTrue(
            self.equivalent(
                "2001:4860:4860::8888",
                53,
                socket.AF_INET6,
                socket.SOCK_DGRAM,
                flags=socket.AI_NUMERICHOST,
            )
        )

    def test_getaddrinfo_nxdomain(self):
        try:
            socket.getaddrinfo("nxdomain.dnspython.org.", 53)
            self.assertTrue(False)  # should not happen!
        except socket.gaierror as e:
            self.assertEqual(e.errno, socket.EAI_NONAME)

    def test_getaddrinfo_service(self):
        a = socket.getaddrinfo("dns.google", "domain")
        b = socket.getaddrinfo("dns.google", 53)
        self.assertTrue(self.equivalent_info(a, b, "dns.google"))
        try:
            socket.getaddrinfo("dns.google", "domain", flags=socket.AI_NUMERICSERV)
            self.assertTrue(False)  # should not happen!
        except socket.gaierror as e:
            self.assertEqual(e.errno, socket.EAI_NONAME)

    def test_getaddrinfo_only_service(self):
        infos = socket.getaddrinfo(
            service=53,
            family=socket.AF_INET,
            socktype=socket.SOCK_DGRAM,
            proto=socket.IPPROTO_UDP,
        )
        self.assertEqual(len(infos), 1)
        info = infos[0]
        self.assertEqual(info[0], socket.AF_INET)
        self.assertEqual(info[1], socket.SOCK_DGRAM)
        self.assertEqual(info[2], socket.IPPROTO_UDP)
        self.assertEqual(info[4], ("127.0.0.1", 53))

    def test_unknown_service_fails(self):
        with self.assertRaises(socket.gaierror):
            socket.getaddrinfo("dns.google.", "bogus-service")

    def test_getnameinfo_tcp(self):
        info = socket.getnameinfo(("8.8.8.8", 53))
        self.assertEqual(info, ("dns.google", "domain"))

    def test_getnameinfo_udp(self):
        info = socket.getnameinfo(("8.8.8.8", 53), socket.NI_DGRAM)
        self.assertEqual(info, ("dns.google", "domain"))

    # Give up on testing this for now as all of the names I've considered
    # using for testing are part of CDNs and there is deep magic in
    # gethostbyaddr() that python's getfqdn() is using.  At any rate,
    # the problem is that dnspython just gives up whereas the native python
    # code is looking up www.dnspython.org, picking a CDN IPv4 address
    # (sometimes) and returning the reverse lookup of that address (i.e.
    # the domain name of the CDN server).  This isn't what I'd consider the
    # FQDN of www.dnspython.org to be!
    #
    #    def test_getfqdn(self):
    #        b = socket.getfqdn('www.dnspython.org')
    #        # we do this now because python's original getfqdn calls
    #        # gethostbyaddr() and we don't want it to call us!
    #        dns.resolver.restore_system_resolver()
    #        a = dns.resolver._original_getfqdn('www.dnspython.org')
    #        self.assertEqual(dns.name.from_text(a), dns.name.from_text(b))

    def test_gethostbyaddr(self):
        a = dns.resolver._original_gethostbyaddr("8.8.8.8")
        b = socket.gethostbyaddr("8.8.8.8")
        # We only test elements 0 and 2 as we don't set aliases currently!
        self.assertEqual(a[0], b[0])
        self.assertEqual(a[2], b[2])
        a = dns.resolver._original_gethostbyaddr("2001:4860:4860::8888")
        b = socket.gethostbyaddr("2001:4860:4860::8888")
        self.assertEqual(a[0], b[0])
        self.assertEqual(a[2], b[2])


class FakeResolver:
    def resolve(self, *args, **kwargs):
        raise dns.exception.Timeout


class OverrideSystemResolverUsingFakeResolverTestCase(unittest.TestCase):
    def setUp(self):
        self.res = FakeResolver()
        dns.resolver.override_system_resolver(self.res)

    def tearDown(self):
        dns.resolver.restore_system_resolver()
        self.res = None

    def test_temporary_failure(self):
        with self.assertRaises(socket.gaierror):
            socket.getaddrinfo("dns.google")

    # We don't need the fake resolver for the following tests, but we
    # don't need the live network either, so we're testing here.

    def test_no_host_or_service_fails(self):
        with self.assertRaises(socket.gaierror):
            socket.getaddrinfo()

    def test_AI_ADDRCONFIG_fails(self):
        with self.assertRaises(socket.gaierror):
            socket.getaddrinfo("dns.google", flags=socket.AI_ADDRCONFIG)

    def test_gethostbyaddr_of_name_fails(self):
        with self.assertRaises(socket.gaierror):
            socket.gethostbyaddr("bogus")


@unittest.skipIf(not tests.util.is_internet_reachable(), "Internet not reachable")
class OverrideSystemResolverUsingDefaultResolverTestCase(unittest.TestCase):
    def setUp(self):
        self.res = FakeResolver()
        dns.resolver.override_system_resolver()

    def tearDown(self):
        dns.resolver.restore_system_resolver()
        self.res = None

    def test_override(self):
        self.assertEqual(dns.resolver._resolver, dns.resolver.default_resolver)



#!/usr/bin/env python3

# Using Discovery of Designated Resolvers (synchronous I/O)

import dns.resolver

res = dns.resolver.Resolver(configure=False)
res.nameservers = ["1.1.1.1"]
# Invoke try_ddr() to attempt to upgrade the connection via DDR
res.try_ddr()
# Do a sample resolution
for rr in res.resolve("www.google.com", "A"):
    print(rr.address)
# Note that the nameservers have been upgraded
print(res.nameservers)


# Using Discovery of Designated Resolvers (asynchronous I/O)

# We show using asyncio, but if you comment out asyncio lines
# and uncomment the trio lines, it will work with trio too.

import asyncio

# import trio

import dns.asyncresolver


async def amain():
    res = dns.asyncresolver.Resolver(configure=False)
    res.nameservers = ["8.8.8.8"]
    await res.try_ddr()

    for rr in await res.resolve("www.google.com", "A"):
        print(rr.address)

    print(res.nameservers)


asyncio.run(amain())
# trio.run(amain)



#!/usr/bin/env python3

import dns.edns
import dns.message
import dns.query
import dns.resolver

n = "."
t = dns.rdatatype.SOA
l = "google.com"  # Address of l.root-servers.net, '199.7.83.42'
i = "ns1.isc.org"  # Address of ns1.isc.org, for COOKIEs, '149.20.1.73'

o_list = []

# A query without options
o_list.append((l, dict()))

# The same query, but with empty options list
o_list.append((l, dict(options=[])))

# Use use_edns() to specify EDNS0 options, such as buffer size
o_list.append((l, dict(payload=2000)))

# With an NSID option, but with use_edns() to specify the options
edns_kwargs = dict(
    edns=0, options=[dns.edns.GenericOption(dns.edns.OptionType.NSID, b"")]
)
o_list.append((l, edns_kwargs))

# With a COOKIE
o_list.append(
    (
        i,
        dict(
            options=[
                dns.edns.GenericOption(
                    dns.edns.OptionType.COOKIE, b"0xfe11ac99bebe3322"
                )
            ]
        ),
    )
)

# With an ECS option using cloudflare dns address
o_list.append((l, dict(options=[dns.edns.ECSOption("1.1.1.1", 24)])))

# With an ECS option using the current machine address
import urllib.request

external_ip = urllib.request.urlopen("https://ident.me").read().decode("utf8")

o_list.append((l, dict(options=[dns.edns.ECSOption(external_ip, 24)])))

aresolver = dns.resolver.Resolver()

for addr, edns_kwargs in o_list:
    if edns_kwargs:
        aresolver.use_edns(**edns_kwargs)
    aresolver.nameservers = ["8.8.8.8"]
    print(list(aresolver.resolve(addr, "A")))



#!/usr/bin/python
import argparse
import sys

import dns.exception
import dns.message


def main():
    parser = argparse.ArgumentParser(
        description="Read sequence of DNS wire formats prefixed with 2-byte "
        "length field - like from TCP socket - and print the messages. This "
        "format is used e.g. by dnsperf -B option and dnsgen."
    )
    parser.add_argument("infile")
    args = parser.parse_args()
    ok_msgs = 0
    bad_msgs = 0

    with open(args.infile, "rb") as infile:
        while True:
            offset = infile.tell()
            len_wire = infile.read(2)
            if len(len_wire) == 0:  # end of file - expected
                break
            if len(len_wire) == 1:
                raise ValueError("incomplete length preamble, offset", offset)
            len_msg = int.from_bytes(len_wire, byteorder="big", signed=False)
            print(f"; msg offset 0x{offset + 2:x}, length {len_msg} bytes")
            msg_wire = infile.read(len_msg)
            if len(msg_wire) != len_msg:
                raise ValueError(
                    f"incomplete message: expected {len_msg} != got {len(msg_wire)}, "
                    f"length field offset 0x{offset:x}",
                )
            try:
                msg = dns.message.from_wire(msg_wire)
                ok_msgs += 1
                print(msg)
            except dns.exception.DNSException as ex:
                print(f"; invalid message, skipping: {ex}")
                bad_msgs += 1
        print(f"; read {ok_msgs} valid and {bad_msgs} invalid messages")
        if bad_msgs:
            sys.exit(1)


if __name__ == "__main__":
    main()



# -*- coding: utf-8
# Copyright (C) Dnspython Contributors, see LICENSE for text of ISC license

# Copyright (C) 2003-2007, 2009-2011 Nominum, Inc.
#
# Permission to use, copy, modify, and distribute this software and its
# documentation for any purpose with or without fee is hereby granted,
# provided that the above copyright notice and this permission notice
# appear in all copies.
#
# THE SOFTWARE IS PROVIDED "AS IS" AND NOMINUM DISCLAIMS ALL WARRANTIES
# WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
# MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL NOMINUM BE LIABLE FOR
# ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
# WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
# ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT
# OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.

import unittest

import dns.name
import dns.rrset


class RRsetTestCase(unittest.TestCase):
    def testEqual1(self):
        r1 = dns.rrset.from_text("foo", 300, "in", "a", "10.0.0.1", "10.0.0.2")
        r2 = dns.rrset.from_text("FOO", 300, "in", "a", "10.0.0.2", "10.0.0.1")
        self.assertEqual(r1, r2)

    def testEqual2(self):
        r1 = dns.rrset.from_text("foo", 300, "in", "a", "10.0.0.1", "10.0.0.2")
        r2 = dns.rrset.from_text("FOO", 600, "in", "a", "10.0.0.2", "10.0.0.1")
        self.assertEqual(r1, r2)

    def testNotEqual1(self):
        r1 = dns.rrset.from_text("fooa", 30, "in", "a", "10.0.0.1", "10.0.0.2")
        r2 = dns.rrset.from_text("FOO", 30, "in", "a", "10.0.0.2", "10.0.0.1")
        self.assertNotEqual(r1, r2)

    def testNotEqual2(self):
        r1 = dns.rrset.from_text("foo", 30, "in", "a", "10.0.0.1", "10.0.0.3")
        r2 = dns.rrset.from_text("FOO", 30, "in", "a", "10.0.0.2", "10.0.0.1")
        self.assertNotEqual(r1, r2)

    def testNotEqual3(self):
        r1 = dns.rrset.from_text(
            "foo", 30, "in", "a", "10.0.0.1", "10.0.0.2", "10.0.0.3"
        )
        r2 = dns.rrset.from_text("FOO", 30, "in", "a", "10.0.0.2", "10.0.0.1")
        self.assertNotEqual(r1, r2)

    def testNotEqual4(self):
        r1 = dns.rrset.from_text("foo", 30, "in", "a", "10.0.0.1")
        r2 = dns.rrset.from_text("FOO", 30, "in", "a", "10.0.0.2", "10.0.0.1")
        self.assertNotEqual(r1, r2)

    def testCodec2003(self):
        r1 = dns.rrset.from_text_list(
            "Königsgäßchen", 30, "in", "ns", ["Königsgäßchen"]
        )
        r2 = dns.rrset.from_text_list(
            "xn--knigsgsschen-lcb0w", 30, "in", "ns", ["xn--knigsgsschen-lcb0w"]
        )
        self.assertEqual(r1, r2)

    @unittest.skipUnless(
        dns.name.have_idna_2008, "Python idna cannot be imported; no IDNA2008"
    )
    def testCodec2008(self):
        r1 = dns.rrset.from_text_list(
            "Königsgäßchen",
            30,
            "in",
            "ns",
            ["Königsgäßchen"],
            idna_codec=dns.name.IDNA_2008,
        )
        r2 = dns.rrset.from_text_list(
            "xn--knigsgchen-b4a3dun",
            30,
            "in",
            "ns",
            ["xn--knigsgchen-b4a3dun"],
            idna_codec=dns.name.IDNA_2008,
        )
        self.assertEqual(r1, r2)

    def testCopy(self):
        r1 = dns.rrset.from_text_list("foo", 30, "in", "a", ["10.0.0.1", "10.0.0.2"])
        r2 = r1.copy()
        self.assertFalse(r1 is r2)
        self.assertTrue(r1 == r2)

    def testFullMatch1(self):
        r1 = dns.rrset.from_text_list("foo", 30, "in", "a", ["10.0.0.1", "10.0.0.2"])
        self.assertTrue(
            r1.full_match(
                r1.name, dns.rdataclass.IN, dns.rdatatype.A, dns.rdatatype.NONE
            )
        )

    def testFullMatch2(self):
        r1 = dns.rrset.from_text_list("foo", 30, "in", "a", ["10.0.0.1", "10.0.0.2"])
        r1.deleting = dns.rdataclass.NONE
        self.assertTrue(
            r1.full_match(
                r1.name,
                dns.rdataclass.IN,
                dns.rdatatype.A,
                dns.rdatatype.NONE,
                dns.rdataclass.NONE,
            )
        )

    def testNoFullMatch1(self):
        n = dns.name.from_text("bar", None)
        r1 = dns.rrset.from_text_list("foo", 30, "in", "a", ["10.0.0.1", "10.0.0.2"])
        self.assertFalse(
            r1.full_match(
                n,
                dns.rdataclass.IN,
                dns.rdatatype.A,
                dns.rdatatype.NONE,
                dns.rdataclass.ANY,
            )
        )

    def testNoFullMatch2(self):
        r1 = dns.rrset.from_text_list("foo", 30, "in", "a", ["10.0.0.1", "10.0.0.2"])
        r1.deleting = dns.rdataclass.NONE
        self.assertFalse(
            r1.full_match(
                r1.name,
                dns.rdataclass.IN,
                dns.rdatatype.A,
                dns.rdatatype.NONE,
                dns.rdataclass.ANY,
            )
        )

    def testNoFullMatch3(self):
        r1 = dns.rrset.from_text_list("foo", 30, "in", "a", ["10.0.0.1", "10.0.0.2"])
        self.assertFalse(
            r1.full_match(
                r1.name,
                dns.rdataclass.IN,
                dns.rdatatype.MX,
                dns.rdatatype.NONE,
                dns.rdataclass.ANY,
            )
        )

    def testMatchCompatibilityWithFullMatch(self):
        r1 = dns.rrset.from_text_list("foo", 30, "in", "a", ["10.0.0.1", "10.0.0.2"])
        self.assertTrue(
            r1.match(r1.name, dns.rdataclass.IN, dns.rdatatype.A, dns.rdatatype.NONE)
        )

    def testMatchCompatibilityWithRdatasetMatch(self):
        r1 = dns.rrset.from_text_list("foo", 30, "in", "a", ["10.0.0.1", "10.0.0.2"])
        self.assertTrue(
            r1.match(dns.rdataclass.IN, dns.rdatatype.A, dns.rdatatype.NONE)
        )

    def testToRdataset(self):
        r1 = dns.rrset.from_text_list("foo", 30, "in", "a", ["10.0.0.1", "10.0.0.2"])
        r2 = dns.rdataset.from_text_list("in", "a", 30, ["10.0.0.1", "10.0.0.2"])
        self.assertEqual(r1.to_rdataset(), r2)

    def testFromRdata(self):
        rdata1 = dns.rdata.from_text(dns.rdataclass.IN, dns.rdatatype.A, "10.0.0.1")
        rdata2 = dns.rdata.from_text(dns.rdataclass.IN, dns.rdatatype.A, "10.0.0.2")
        expected_rrs = dns.rrset.from_text(
            "foo", 300, "in", "a", "10.0.0.1", "10.0.0.2"
        )
        rrs = dns.rrset.from_rdata("foo", 300, rdata1, rdata2)
        self.assertEqual(rrs, expected_rrs)

    def testEmptyList(self):
        def bad():
            rrs = dns.rrset.from_rdata_list("foo", 300, [])

        self.assertRaises(ValueError, bad)

    def testTTLMinimization(self):
        rrs = dns.rrset.RRset(
            dns.name.from_text("foo"), dns.rdataclass.IN, dns.rdatatype.A
        )
        rdata1 = dns.rdata.from_text(dns.rdataclass.IN, dns.rdatatype.A, "10.0.0.1")
        rdata2 = dns.rdata.from_text(dns.rdataclass.IN, dns.rdatatype.A, "10.0.0.2")
        rrs.add(rdata1, 300)
        self.assertEqual(rrs.ttl, 300)
        rrs.add(rdata2, 30)
        self.assertEqual(rrs.ttl, 30)
        # adding the same thing with a smaller TTL also minimizes
        rrs.add(rdata2, 3)
        self.assertEqual(rrs.ttl, 3)

    def testNotEqualOtherType(self):
        rrs = dns.rrset.RRset(
            dns.name.from_text("foo"), dns.rdataclass.IN, dns.rdatatype.A
        )
        self.assertFalse(rrs == 123)

    def testRepr(self):
        rrset = dns.rrset.from_text("foo", 30, "in", "a", "10.0.0.1", "10.0.0.2")
        self.assertEqual(repr(rrset), "<DNS foo IN A RRset: [<10.0.0.1>, <10.0.0.2>]>")
        rrset.deleting = dns.rdataclass.NONE
        self.assertEqual(
            repr(rrset),
            "<DNS foo IN A delete=NONE RRset: " + "[<10.0.0.1>, <10.0.0.2>]>",
        )
        rrset = dns.rrset.from_text(
            "foo",
            30,
            "in",
            "rrsig",
            "A 1 3 3600 20200701000000 20200601000000 1 NAME Ym9ndXM=",
        )
        self.assertEqual(
            repr(rrset),
            "<DNS foo IN RRSIG(A) RRset: "
            + "[<A 1 3 3600 20200701000000 20200601000000 "
            + "1 NAME Ym9ndXM=>]>",
        )


if __name__ == "__main__":
    unittest.main()



# Copyright (C) Dnspython Contributors, see LICENSE for text of ISC license

# Copyright (C) 2011 Nominum, Inc.
#
# Permission to use, copy, modify, and distribute this software and its
# documentation for any purpose with or without fee is hereby granted,
# provided that the above copyright notice and this permission notice
# appear in all copies.
#
# THE SOFTWARE IS PROVIDED "AS IS" AND NOMINUM DISCLAIMS ALL WARRANTIES
# WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
# MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL NOMINUM BE LIABLE FOR
# ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
# WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
# ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT
# OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.

import os
import unittest

import dns.dnssec
import dns.exception
from dns.dnssectypes import Algorithm
from dns.rdtypes.ANY.DNSKEY import DNSKEY

try:
    from dns.dnssecalgs import (
        get_algorithm_cls,
        get_algorithm_cls_from_dnskey,
        register_algorithm_cls,
    )
    from dns.dnssecalgs.dsa import PrivateDSA, PrivateDSANSEC3SHA1
    from dns.dnssecalgs.ecdsa import PrivateECDSAP256SHA256, PrivateECDSAP384SHA384
    from dns.dnssecalgs.eddsa import PrivateED448, PrivateED25519, PublicED25519
    from dns.dnssecalgs.rsa import (
        PrivateRSAMD5,
        PrivateRSASHA1,
        PrivateRSASHA1NSEC3SHA1,
        PrivateRSASHA256,
        PrivateRSASHA512,
    )
except ImportError:
    pass  # Cryptography ImportError already handled in dns.dnssec


@unittest.skipUnless(dns.dnssec._have_pyca, "Python Cryptography cannot be imported")
class DNSSECAlgorithm(unittest.TestCase):
    def _test_dnssec_alg(self, private_cls, key_size=None):
        public_cls = private_cls.public_cls

        private_key = (
            private_cls.generate(key_size) if key_size else private_cls.generate()
        )

        # sign random data
        data = os.urandom(1024)
        signature = private_key.sign(data, verify=True)

        # validate signature using public key
        public_key = private_key.public_key()
        public_key.verify(signature, data)

        # create DNSKEY
        dnskey = public_key.to_dnskey()
        dnskey2 = public_cls.from_dnskey(dnskey).to_dnskey()
        self.assertEqual(dnskey, dnskey2)

        # test cryptography keys
        _ = private_cls(key=private_key.key)
        _ = public_cls(key=public_key.key)

        # to/from PEM
        password = b"mekmitasdigoat"
        private_pem = private_key.to_pem()
        private_pem_encrypted = private_key.to_pem(password=password)
        public_pem = public_key.to_pem()
        _ = private_cls.from_pem(private_pem)
        _ = private_cls.from_pem(private_pem_encrypted, password)
        _ = public_cls.from_pem(public_pem)

    def test_rsa(self):
        self._test_dnssec_alg(PrivateRSAMD5, 2048)
        self._test_dnssec_alg(PrivateRSASHA1, 2048)
        self._test_dnssec_alg(PrivateRSASHA1NSEC3SHA1, 2048)
        self._test_dnssec_alg(PrivateRSASHA256, 2048)
        self._test_dnssec_alg(PrivateRSASHA512, 2048)

    def test_dsa(self):
        self._test_dnssec_alg(PrivateDSA, 1024)
        self._test_dnssec_alg(PrivateDSANSEC3SHA1, 1024)
        with self.assertRaises(ValueError):
            k = PrivateDSA.generate(2048)
            k.sign(b"hello")

    def test_ecdsa(self):
        self._test_dnssec_alg(PrivateECDSAP256SHA256)
        self._test_dnssec_alg(PrivateECDSAP384SHA384)

    def test_eddsa(self):
        self._test_dnssec_alg(PrivateED25519)
        self._test_dnssec_alg(PrivateED448)

    def test_algorithm_mismatch(self):
        private_key_ed448 = PrivateED448.generate()
        dnskey_ed448 = private_key_ed448.public_key().to_dnskey()
        with self.assertRaises(dns.exception.AlgorithmKeyMismatch):
            PublicED25519.from_dnskey(dnskey_ed448)


@unittest.skipUnless(dns.dnssec._have_pyca, "Python Cryptography cannot be imported")
class DNSSECAlgorithmPrivateAlgorithm(unittest.TestCase):
    def test_private(self):
        class PublicExampleAlgorithm(PublicED25519):
            algorithm = Algorithm.PRIVATEDNS
            name = dns.name.from_text("algorithm.example.com")

            def encode_key_bytes(self) -> bytes:
                return self.name.to_wire() + super().encode_key_bytes()

            @classmethod
            def from_dnskey(cls, key: DNSKEY) -> "PublicEDDSA":
                return cls(
                    key=cls.key_cls.from_public_bytes(
                        key.key[len(cls.name.to_wire()) :]
                    ),
                )

        class PrivateExampleAlgorithm(PrivateED25519):
            public_cls = PublicExampleAlgorithm

        register_algorithm_cls(
            algorithm=Algorithm.PRIVATEDNS,
            algorithm_cls=PrivateExampleAlgorithm,
            name=PublicExampleAlgorithm.name,
        )

        private_key = PrivateExampleAlgorithm.generate()
        public_key = private_key.public_key()

        name = dns.name.from_text("example.com")
        rdataset = dns.rdataset.from_text_list("in", "a", 30, ["10.0.0.1", "10.0.0.2"])
        rrset = (name, rdataset)
        ttl = 60
        lifetime = 3600
        rrname = rrset[0]
        signer = rrname
        dnskey = dns.dnssec.make_dnskey(
            public_key=public_key, algorithm=Algorithm.PRIVATEDNS
        )
        dnskey_rrset = dns.rrset.from_rdata(signer, ttl, dnskey)

        rrsig = dns.dnssec.sign(
            rrset=rrset,
            private_key=private_key,
            dnskey=dnskey,
            lifetime=lifetime,
            signer=signer,
            verify=True,
            policy=None,
        )

        keys = {signer: dnskey_rrset}
        rrsigset = dns.rrset.from_rdata(rrname, ttl, rrsig)
        dns.dnssec.validate(rrset=rrset, rrsigset=rrsigset, keys=keys, policy=None)

    def test_register(self):
        register_algorithm_cls(
            algorithm=Algorithm.PRIVATEDNS,
            algorithm_cls=PrivateED25519,
            name="ed25519.example.com",
        )
        register_algorithm_cls(
            algorithm=Algorithm.PRIVATEOID,
            algorithm_cls=PrivateED448,
            oid=bytes([1, 2, 3, 4]),
        )
        register_algorithm_cls(
            algorithm=251,
            algorithm_cls=PrivateED25519,
        )

        with self.assertRaises(TypeError):
            register_algorithm_cls(algorithm=251, algorithm_cls=str, name="example.com")

        with self.assertRaises(ValueError):
            register_algorithm_cls(
                algorithm=251, algorithm_cls=PrivateED25519, name="example.com"
            )

        with self.assertRaises(ValueError):
            register_algorithm_cls(
                algorithm=251, algorithm_cls=PrivateED25519, oid=bytes([1, 2, 3, 4])
            )

        with self.assertRaises(ValueError):
            register_algorithm_cls(
                algorithm=Algorithm.PRIVATEDNS,
                algorithm_cls=PrivateED25519,
                oid=bytes([1, 2, 3, 4]),
            )

        with self.assertRaises(ValueError):
            register_algorithm_cls(
                algorithm=Algorithm.PRIVATEOID,
                algorithm_cls=PrivateED25519,
                name="example.com",
            )

        dnskey_251 = DNSKEY(
            "IN",
            "DNSKEY",
            256,
            3,
            251,
            b"hello",
        )
        dnskey_dns = DNSKEY(
            "IN",
            "DNSKEY",
            256,
            3,
            Algorithm.PRIVATEDNS,
            dns.name.from_text("ed25519.example.com").to_wire() + b"hello",
        )
        dnskey_dns_unknown = DNSKEY(
            "IN",
            "DNSKEY",
            256,
            3,
            Algorithm.PRIVATEDNS,
            dns.name.from_text("unknown.example.com").to_wire() + b"hello",
        )
        dnskey_oid = DNSKEY(
            "IN",
            "DNSKEY",
            256,
            3,
            Algorithm.PRIVATEOID,
            bytes([4, 1, 2, 3, 4]) + b"hello",
        )
        dnskey_oid_unknown = DNSKEY(
            "IN",
            "DNSKEY",
            256,
            3,
            Algorithm.PRIVATEOID,
            bytes([4, 42, 42, 42, 42]) + b"hello",
        )

        with self.assertRaises(dns.exception.UnsupportedAlgorithm):
            _ = get_algorithm_cls(250)

        algorithm_cls = get_algorithm_cls(251)
        self.assertEqual(algorithm_cls, PrivateED25519)

        algorithm_cls = get_algorithm_cls_from_dnskey(dnskey_251)
        self.assertEqual(algorithm_cls, PrivateED25519)

        algorithm_cls = get_algorithm_cls_from_dnskey(dnskey_dns)
        self.assertEqual(algorithm_cls, PrivateED25519)

        with self.assertRaises(dns.exception.UnsupportedAlgorithm):
            _ = get_algorithm_cls_from_dnskey(dnskey_dns_unknown)

        algorithm_cls = get_algorithm_cls_from_dnskey(dnskey_oid)
        self.assertEqual(algorithm_cls, PrivateED448)

        with self.assertRaises(dns.exception.UnsupportedAlgorithm):
            _ = get_algorithm_cls_from_dnskey(dnskey_oid_unknown)

    def test_register_canonical_lookup(self):
        register_algorithm_cls(
            algorithm=Algorithm.PRIVATEDNS,
            algorithm_cls=PrivateED25519,
            name="testing1234.example.com",
        )

        dnskey_dns = DNSKEY(
            "IN",
            "DNSKEY",
            256,
            3,
            Algorithm.PRIVATEDNS,
            dns.name.from_text("TESTING1234.EXAMPLE.COM").to_wire() + b"hello",
        )

        algorithm_cls = get_algorithm_cls_from_dnskey(dnskey_dns)
        self.assertEqual(algorithm_cls, PrivateED25519)

    def test_register_private_without_prefix(self):
        with self.assertRaises(ValueError):
            register_algorithm_cls(
                algorithm=Algorithm.PRIVATEDNS,
                algorithm_cls=PrivateED25519,
            )
        with self.assertRaises(ValueError):
            register_algorithm_cls(
                algorithm=Algorithm.PRIVATEOID,
                algorithm_cls=PrivateED25519,
            )


if __name__ == "__main__":
    unittest.main()



import pytest

import dns.rrset
from dns.zonefile import read_rrsets

expected_mx_1 = dns.rrset.from_text("name.", 300, "in", "mx", "10 a.", "20 b.")
expected_mx_2 = dns.rrset.from_text("name.", 10, "in", "mx", "10 a.", "20 b.")
expected_mx_3 = dns.rrset.from_text("foo.", 10, "in", "mx", "10 a.")
expected_mx_4 = dns.rrset.from_text("bar.", 10, "in", "mx", "20 b.")
expected_mx_5 = dns.rrset.from_text("foo.example.", 10, "in", "mx", "10 a.example.")
expected_mx_6 = dns.rrset.from_text("bar.example.", 10, "in", "mx", "20 b.")
expected_mx_7 = dns.rrset.from_text("foo", 10, "in", "mx", "10 a")
expected_mx_8 = dns.rrset.from_text("bar", 10, "in", "mx", "20 b.")
expected_ns_1 = dns.rrset.from_text("name.", 300, "in", "ns", "hi.")
expected_ns_2 = dns.rrset.from_text("name.", 300, "ch", "ns", "hi.")


def equal_rrsets(a, b):
    # return True iff. a and b have the same rrsets regardless of order
    if len(a) != len(b):
        return False
    for rrset in a:
        if not rrset in b:
            return False
    return True


def test_name_ttl_rdclass_forced():
    input = """;
mx 10 a
mx 20 b.
ns hi"""
    rrsets = read_rrsets(input, name="name", ttl=300)
    assert equal_rrsets(rrsets, [expected_mx_1, expected_ns_1])
    assert rrsets[0].ttl == 300
    assert rrsets[1].ttl == 300


def test_name_ttl_rdclass_forced_rdata_split():
    input = """;
mx 10 a
ns hi
mx 20 b."""
    rrsets = read_rrsets(input, name="name", ttl=300)
    assert equal_rrsets(rrsets, [expected_mx_1, expected_ns_1])


def test_name_ttl_rdclass_rdtype_forced():
    input = """;
10 a
20 b."""
    rrsets = read_rrsets(input, name="name", ttl=300, rdtype="mx")
    assert equal_rrsets(rrsets, [expected_mx_1])


def test_name_rdclass_forced():
    input = """30 mx 10 a
10 mx 20 b.
"""
    rrsets = read_rrsets(input, name="name")
    assert equal_rrsets(rrsets, [expected_mx_2])
    assert rrsets[0].ttl == 10


def test_rdclass_forced():
    input = """;
foo 20 mx 10 a
bar 30 mx 20 b.
"""
    rrsets = read_rrsets(input)
    assert equal_rrsets(rrsets, [expected_mx_3, expected_mx_4])


def test_rdclass_forced_with_origin():
    input = """;
foo 20 mx 10 a
bar.example. 30 mx 20 b.
"""
    rrsets = read_rrsets(input, origin="example")
    assert equal_rrsets(rrsets, [expected_mx_5, expected_mx_6])


def test_rdclass_forced_with_origin_relativized():
    input = """;
foo 20 mx 10 a.example.
bar.example. 30 mx 20 b.
"""
    rrsets = read_rrsets(input, origin="example", relativize=True)
    assert equal_rrsets(rrsets, [expected_mx_7, expected_mx_8])


def test_rdclass_matching_default_tolerated():
    input = """;
foo 20 mx 10 a.example.
bar.example. 30 in mx 20 b.
"""
    rrsets = read_rrsets(input, origin="example", relativize=True, rdclass=None)
    assert equal_rrsets(rrsets, [expected_mx_7, expected_mx_8])


def test_rdclass_not_matching_default_rejected():
    input = """;
foo 20 mx 10 a.example.
bar.example. 30 ch mx 20 b.
"""
    with pytest.raises(dns.exception.SyntaxError):
        rrsets = read_rrsets(input, origin="example", relativize=True, rdclass=None)


def test_default_rdclass_is_none():
    input = ""
    with pytest.raises(TypeError):
        rrsets = read_rrsets(
            input, default_rdclass=None, origin="example", relativize=True
        )


def test_name_rdclass_rdtype_force():
    # No real-world usage should do this, but it can be specified so we test it.
    input = """;
30 10 a
10 20 b.
"""
    rrsets = read_rrsets(input, name="name", rdtype="mx")
    assert equal_rrsets(rrsets, [expected_mx_1])
    assert rrsets[0].ttl == 10


def test_rdclass_rdtype_force():
    # No real-world usage should do this, but it can be specified so we test it.
    input = """;
foo 30 10 a
bar 30 20 b.
"""
    rrsets = read_rrsets(input, rdtype="mx")
    assert equal_rrsets(rrsets, [expected_mx_3, expected_mx_4])


# also weird but legal
# input5 = '''foo 30 10 a
# bar 10 20 foo.
#'''



# -*- coding: utf-8
# Copyright (C) Dnspython Contributors, see LICENSE for text of ISC license

# Copyright (C) 2003-2007, 2009-2011 Nominum, Inc.
#
# Permission to use, copy, modify, and distribute this software and its
# documentation for any purpose with or without fee is hereby granted,
# provided that the above copyright notice and this permission notice
# appear in all copies.
#
# THE SOFTWARE IS PROVIDED "AS IS" AND NOMINUM DISCLAIMS ALL WARRANTIES
# WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
# MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL NOMINUM BE LIABLE FOR
# ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
# WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
# ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT
# OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.

import operator
import struct
import unittest
from io import BytesIO

import dns.edns
import dns.wire


class OptionTestCase(unittest.TestCase):
    def testGenericOption(self):
        opt = dns.edns.GenericOption(3, b"data")
        io = BytesIO()
        opt.to_wire(io)
        data = io.getvalue()
        self.assertEqual(data, b"data")
        self.assertEqual(dns.edns.option_from_wire(3, data, 0, len(data)), opt)
        self.assertEqual(str(opt), "Generic 3")

    def testECSOption_prefix_length(self):
        opt = dns.edns.ECSOption("1.2.255.33", 20)
        io = BytesIO()
        opt.to_wire(io)
        data = io.getvalue()
        self.assertEqual(data, b"\x00\x01\x14\x00\x01\x02\xf0")

    def testECSOption(self):
        opt = dns.edns.ECSOption("1.2.3.4", 24)
        io = BytesIO()
        opt.to_wire(io)
        data = io.getvalue()
        self.assertEqual(data, b"\x00\x01\x18\x00\x01\x02\x03")
        # default srclen
        opt = dns.edns.ECSOption("1.2.3.4")
        io = BytesIO()
        opt.to_wire(io)
        data = io.getvalue()
        self.assertEqual(data, b"\x00\x01\x18\x00\x01\x02\x03")
        self.assertEqual(opt.to_text(), "ECS 1.2.3.4/24 scope/0")

    def testECSOption25(self):
        opt = dns.edns.ECSOption("1.2.3.255", 25)
        io = BytesIO()
        opt.to_wire(io)
        data = io.getvalue()
        self.assertEqual(data, b"\x00\x01\x19\x00\x01\x02\x03\x80")

        opt2 = dns.edns.option_from_wire(dns.edns.ECS, data, 0, len(data))
        self.assertEqual(opt2.otype, dns.edns.ECS)
        self.assertEqual(opt2.address, "1.2.3.128")
        self.assertEqual(opt2.srclen, 25)
        self.assertEqual(opt2.scopelen, 0)

    def testECSOption_v6(self):
        opt = dns.edns.ECSOption("2001:4b98::1")
        io = BytesIO()
        opt.to_wire(io)
        data = io.getvalue()
        self.assertEqual(data, b"\x00\x02\x38\x00\x20\x01\x4b\x98\x00\x00\x00")

        opt2 = dns.edns.option_from_wire(dns.edns.ECS, data, 0, len(data))
        self.assertEqual(opt2.otype, dns.edns.ECS)
        self.assertEqual(opt2.address, "2001:4b98::")
        self.assertEqual(opt2.srclen, 56)
        self.assertEqual(opt2.scopelen, 0)

    def testECSOption_from_text_valid(self):
        ecs1 = dns.edns.ECSOption.from_text("1.2.3.4/24/0")
        self.assertEqual(ecs1, dns.edns.ECSOption("1.2.3.4", 24, 0))

        ecs2 = dns.edns.ECSOption.from_text("1.2.3.4/24")
        self.assertEqual(ecs2, dns.edns.ECSOption("1.2.3.4", 24, 0))

        ecs3 = dns.edns.ECSOption.from_text("ECS 1.2.3.4/24")
        self.assertEqual(ecs3, dns.edns.ECSOption("1.2.3.4", 24, 0))

        ecs4 = dns.edns.ECSOption.from_text("ECS 1.2.3.4/24/32")
        self.assertEqual(ecs4, dns.edns.ECSOption("1.2.3.4", 24, 32))

        ecs5 = dns.edns.ECSOption.from_text("2001:4b98::1/64/56")
        self.assertEqual(ecs5, dns.edns.ECSOption("2001:4b98::1", 64, 56))

        ecs6 = dns.edns.ECSOption.from_text("2001:4b98::1/64")
        self.assertEqual(ecs6, dns.edns.ECSOption("2001:4b98::1", 64, 0))

        ecs7 = dns.edns.ECSOption.from_text("ECS 2001:4b98::1/0")
        self.assertEqual(ecs7, dns.edns.ECSOption("2001:4b98::1", 0, 0))

        ecs8 = dns.edns.ECSOption.from_text("ECS 2001:4b98::1/64/128")
        self.assertEqual(ecs8, dns.edns.ECSOption("2001:4b98::1", 64, 128))

    def testECSOption_from_text_invalid(self):
        with self.assertRaises(ValueError):
            dns.edns.ECSOption.from_text("some random text 1.2.3.4/24/0 24")

        with self.assertRaises(ValueError):
            dns.edns.ECSOption.from_text("1.2.3.4/twentyfour")

        with self.assertRaises(ValueError):
            dns.edns.ECSOption.from_text("BOGUS 1.2.3.4/5/6/7")

        with self.assertRaises(ValueError):
            dns.edns.ECSOption.from_text("1.2.3.4/5/6/7")

        with self.assertRaises(ValueError):
            dns.edns.ECSOption.from_text("1.2.3.4/24/O")  # <-- that's not a zero

        with self.assertRaises(ValueError):
            dns.edns.ECSOption.from_text("")

        with self.assertRaises(ValueError):
            dns.edns.ECSOption.from_text("1.2.3.4/2001:4b98::1/24")

    def testECSOption_from_wire_invalid(self):
        with self.assertRaises(ValueError):
            opt = dns.edns.option_from_wire(
                dns.edns.ECS, b"\x00\xff\x18\x00\x01\x02\x03", 0, 7
            )

    def testEDEOption(self):
        opt = dns.edns.EDEOption(3)
        io = BytesIO()
        opt.to_wire(io)
        data = io.getvalue()
        self.assertEqual(data, b"\x00\x03")
        self.assertEqual(str(opt), "EDE 3 (Stale Answer)")
        # with text
        opt = dns.edns.EDEOption(16, "test")
        io = BytesIO()
        opt.to_wire(io)
        data = io.getvalue()
        self.assertEqual(data, b"\x00\x10test")

    def testEDEOption_invalid(self):
        with self.assertRaises(ValueError):
            opt = dns.edns.EDEOption(-1)
        with self.assertRaises(ValueError):
            opt = dns.edns.EDEOption(65536)
        with self.assertRaises(ValueError):
            opt = dns.edns.EDEOption(0, 0)

    def testEDEOption_from_wire(self):
        data = b"\x00\01"
        self.assertEqual(
            dns.edns.option_from_wire(dns.edns.EDE, data, 0, 2), dns.edns.EDEOption(1)
        )
        data = b"\x00\01test"
        self.assertEqual(
            dns.edns.option_from_wire(dns.edns.EDE, data, 0, 6),
            dns.edns.EDEOption(1, "test"),
        )
        # utf-8 text MAY be null-terminated
        data = b"\x00\01test\x00"
        self.assertEqual(
            dns.edns.option_from_wire(dns.edns.EDE, data, 0, 7),
            dns.edns.EDEOption(1, "test"),
        )

    def test_basic_relations(self):
        o1 = dns.edns.ECSOption.from_text("1.2.3.0/24/0")
        o2 = dns.edns.ECSOption.from_text("1.2.4.0/24/0")
        self.assertTrue(o1 == o1)
        self.assertTrue(o1 != o2)
        self.assertTrue(o1 < o2)
        self.assertTrue(o1 <= o2)
        self.assertTrue(o2 > o1)
        self.assertTrue(o2 >= o1)
        o1 = dns.edns.ECSOption.from_text("1.2.4.0/23/0")
        o2 = dns.edns.ECSOption.from_text("1.2.4.0/24/0")
        self.assertTrue(o1 < o2)
        o1 = dns.edns.ECSOption.from_text("1.2.4.0/24/0")
        o2 = dns.edns.ECSOption.from_text("1.2.4.0/24/1")
        self.assertTrue(o1 < o2)

    def test_incompatible_relations(self):
        o1 = dns.edns.GenericOption(3, b"data")
        o2 = dns.edns.ECSOption.from_text("1.2.3.5/24/0")
        for oper in [operator.lt, operator.le, operator.ge, operator.gt]:
            self.assertRaises(TypeError, lambda: oper(o1, o2))
        self.assertFalse(o1 == o2)
        self.assertTrue(o1 != o2)
        self.assertFalse(o1 == 123)
        self.assertTrue(o1 != 123)

    def testNSIDOption(self):
        opt = dns.edns.NSIDOption(b"testing")
        io = BytesIO()
        opt.to_wire(io)
        data = io.getvalue()
        self.assertEqual(data, b"testing")
        self.assertEqual(str(opt), "NSID testing")
        opt = dns.edns.NSIDOption(b"\xfe\xff")
        io = BytesIO()
        opt.to_wire(io)
        data = io.getvalue()
        self.assertEqual(data, b"\xfe\xff")
        self.assertEqual(str(opt), "NSID feff")
        o = dns.edns.option_from_wire(dns.edns.OptionType.NSID, data, 0, len(data))
        self.assertEqual(o.nsid, b"\xfe\xff")

    def test_option_registration(self):
        U32OptionType = 9999

        class U32Option(dns.edns.Option):
            def __init__(self, value=None):
                super().__init__(U32OptionType)
                self.value = value

            def to_wire(self, file=None):
                data = struct.pack("!I", self.value)
                if file:
                    file.write(data)
                else:
                    return data

            @classmethod
            def from_wire_parser(cls, otype, parser):
                (value,) = parser.get_struct("!I")
                return cls(value)

        try:
            dns.edns.register_type(U32Option, U32OptionType)
            generic = dns.edns.GenericOption(U32OptionType, b"\x00\x00\x00\x01")
            wire1 = generic.to_wire()
            u32 = dns.edns.option_from_wire_parser(
                U32OptionType, dns.wire.Parser(wire1)
            )
            self.assertEqual(u32.value, 1)
            wire2 = u32.to_wire()
            self.assertEqual(wire1, wire2)
            self.assertEqual(u32, generic)
        finally:
            dns.edns._type_to_class.pop(U32OptionType, None)

        opt = dns.edns.option_from_wire_parser(9999, dns.wire.Parser(wire1))
        self.assertEqual(opt, generic)



# Copyright (C) Dnspython Contributors, see LICENSE for text of ISC license

# Copyright (C) 2003-2007, 2009-2011 Nominum, Inc.
#
# Permission to use, copy, modify, and distribute this software and its
# documentation for any purpose with or without fee is hereby granted,
# provided that the above copyright notice and this permission notice
# appear in all copies.
#
# THE SOFTWARE IS PROVIDED "AS IS" AND NOMINUM DISCLAIMS ALL WARRANTIES
# WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
# MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL NOMINUM BE LIABLE FOR
# ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
# WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
# ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT
# OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
import random
import socket
import unittest

try:
    import ssl

    _have_ssl = True
except Exception:
    _have_ssl = False

import dns.edns
import dns.message
import dns.query
import dns.rdatatype
import dns.resolver

if dns.query._have_httpx:
    import httpx

import tests.util

resolver_v4_addresses = []
resolver_v6_addresses = []
family = socket.AF_UNSPEC
if tests.util.have_ipv4():
    resolver_v4_addresses = [
        "1.1.1.1",
        "8.8.8.8",
        # '9.9.9.9',
    ]
    family = socket.AF_INET
if tests.util.have_ipv6():
    resolver_v6_addresses = [
        "2606:4700:4700::1111",
        "2001:4860:4860::8888",
        # '2620:fe::fe',
    ]
    if family == socket.AF_INET:
        # we have both working, go back to UNSPEC
        family = socket.AF_UNSPEC
    else:
        # v6 only
        family = socket.AF_INET6

KNOWN_ANYCAST_DOH_RESOLVER_URLS = [
    "https://cloudflare-dns.com/dns-query",
    "https://dns.google/dns-query",
    # 'https://dns11.quad9.net/dns-query',
]

KNOWN_PAD_AWARE_DOH_RESOLVER_URLS = [
    "https://cloudflare-dns.com/dns-query",
    "https://dns.google/dns-query",
]


@unittest.skipUnless(
    dns.query._have_httpx and tests.util.is_internet_reachable() and _have_ssl,
    "Python httpx cannot be imported; no DNS over HTTPS (DOH)",
)
class DNSOverHTTPSTestCaseHttpx(unittest.TestCase):
    def setUp(self):
        self.session = httpx.Client(http1=True, http2=True, verify=True)

    def tearDown(self):
        self.session.close()

    def test_get_request(self):
        nameserver_url = random.choice(KNOWN_ANYCAST_DOH_RESOLVER_URLS)
        q = dns.message.make_query("example.com.", dns.rdatatype.A)
        r = dns.query.https(
            q,
            nameserver_url,
            session=self.session,
            post=False,
            timeout=4,
            family=family,
        )
        self.assertTrue(q.is_response(r))

    def test_post_request(self):
        nameserver_url = random.choice(KNOWN_ANYCAST_DOH_RESOLVER_URLS)
        q = dns.message.make_query("example.com.", dns.rdatatype.A)
        r = dns.query.https(
            q,
            nameserver_url,
            session=self.session,
            post=True,
            timeout=4,
            family=family,
        )
        self.assertTrue(q.is_response(r))

    def test_build_url_from_ip(self):
        self.assertTrue(resolver_v4_addresses or resolver_v6_addresses)
        if resolver_v4_addresses:
            nameserver_ip = random.choice(resolver_v4_addresses)
            q = dns.message.make_query("example.com.", dns.rdatatype.A)
            # For some reason Google's DNS over HTTPS fails when you POST to
            # https://8.8.8.8/dns-query
            # So we're just going to do GET requests here
            r = dns.query.https(
                q, nameserver_ip, session=self.session, post=False, timeout=4
            )

            self.assertTrue(q.is_response(r))
        if resolver_v6_addresses:
            nameserver_ip = random.choice(resolver_v6_addresses)
            q = dns.message.make_query("example.com.", dns.rdatatype.A)
            r = dns.query.https(
                q, nameserver_ip, session=self.session, post=False, timeout=4
            )
            self.assertTrue(q.is_response(r))

    # This test is temporarily disabled as there's an expired certificate issue on one
    # of the servers, so it fails on the part that is supposed to succeed (2023-07-15).

    # def test_bootstrap_address_fails(self):
    #     # We test this to see if v4 is available
    #     if resolver_v4_addresses:
    #         ip = "185.228.168.168"
    #         invalid_tls_url = "https://{}/doh/family-filter/".format(ip)
    #         valid_tls_url = "https://doh.cleanbrowsing.org/doh/family-filter/"
    #         q = dns.message.make_query("example.com.", dns.rdatatype.A)
    #         # make sure CleanBrowsing's IP address will fail TLS certificate
    #         # check.
    #         with self.assertRaises(httpx.ConnectError):
    #             dns.query.https(q, invalid_tls_url, session=self.session, timeout=4)
    #         # And if we don't mangle the URL, it should work.
    #         r = dns.query.https(
    #             q,
    #             valid_tls_url,
    #             session=self.session,
    #             bootstrap_address=ip,
    #             timeout=4,
    #         )
    #         self.assertTrue(q.is_response(r))

    def test_new_session(self):
        nameserver_url = random.choice(KNOWN_ANYCAST_DOH_RESOLVER_URLS)
        q = dns.message.make_query("example.com.", dns.rdatatype.A)
        r = dns.query.https(q, nameserver_url, timeout=4)
        self.assertTrue(q.is_response(r))

    def test_resolver(self):
        res = dns.resolver.Resolver(configure=False)
        res.nameservers = ["https://dns.google/dns-query"]
        answer = res.resolve("dns.google", "A")
        seen = set([rdata.address for rdata in answer])
        self.assertTrue("8.8.8.8" in seen)
        self.assertTrue("8.8.4.4" in seen)

    def test_padded_get(self):
        nameserver_url = random.choice(KNOWN_PAD_AWARE_DOH_RESOLVER_URLS)
        q = dns.message.make_query("example.com.", dns.rdatatype.A, use_edns=0, pad=128)
        r = dns.query.https(
            q, nameserver_url, session=self.session, post=False, timeout=4
        )
        self.assertTrue(q.is_response(r))
        # the response should have a padding option
        self.assertIsNotNone(r.opt)
        has_pad = False
        for o in r.opt[0].options:
            if o.otype == dns.edns.OptionType.PADDING:
                has_pad = True
        self.assertTrue(has_pad)


if __name__ == "__main__":
    unittest.main()



import pickle
import sys

import dns.rdata
import dns.version

# Generate a pickled mx RR for the current dnspython version

mx = dns.rdata.from_text("in", "mx", "10 mx.example.")
filename = f"pickled-{dns.version.MAJOR}-{dns.version.MINOR}.pickle"
with open(filename, "wb") as f:
    pickle.dump(mx, f)
with open(filename, "rb") as f:
    mx2 = pickle.load(f)
if mx == mx2:
    print("ok")
else:
    print("DIFFERENT!")
    sys.exit(1)



# Copyright (C) Dnspython Contributors, see LICENSE for text of ISC license

import asyncio
import sys

import pytest

import dns.asyncbackend
import dns.asyncquery
import dns.message
import dns.query
import dns.rcode

from .util import have_ipv4, have_ipv6, here

try:
    from .nanoquic import Server

    _nanoquic_available = True
except ImportError:
    _nanoquic_available = False

    class Server(object):
        pass


addresses = []
if have_ipv4():
    addresses.append("127.0.0.1")
if have_ipv6():
    addresses.append("::1")
if len(addresses) == 0:
    # no networking
    _nanoquic_available = False


@pytest.mark.skipif(not _nanoquic_available, reason="requires aioquic")
def test_basic_sync():
    q = dns.message.make_query("www.example.", "A")
    for address in addresses:
        with Server(address) as server:
            r = dns.query.quic(q, address, port=server.port, verify=here("tls/ca.crt"))
            assert r.rcode() == dns.rcode.REFUSED


async def amain(address, port):
    q = dns.message.make_query("www.example.", "A")
    r = await dns.asyncquery.quic(q, address, port=port, verify=here("tls/ca.crt"))
    assert r.rcode() == dns.rcode.REFUSED


@pytest.mark.skipif(not _nanoquic_available, reason="requires aioquic")
def test_basic_asyncio():
    dns.asyncbackend.set_default_backend("asyncio")
    for address in addresses:
        with Server(address) as server:
            asyncio.run(amain(address, server.port))


try:
    import trio

    @pytest.mark.skipif(not _nanoquic_available, reason="requires aioquic")
    def test_basic_trio():
        dns.asyncbackend.set_default_backend("trio")
        for address in addresses:
            with Server(address) as server:
                trio.run(amain, address, server.port)

except ImportError:
    pass



# Copyright (C) 2014 Red Hat, Inc.
# Author: Petr Spacek <pspacek@redhat.com>
#
# Permission to use, copy, modify, and distribute this software and its
# documentation for any purpose with or without fee is hereby granted,
# provided that the above copyright notice and this permission notice
# appear in all copies.
#
# THE SOFTWARE IS PROVIDED 'AS IS' AND RED HAT DISCLAIMS ALL WARRANTIES
# WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
# MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL NOMINUM BE LIABLE FOR
# ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
# WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
# ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT
# OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.

import unittest

import dns.rrset
import dns.rdtypes.ANY.LOC


class RdtypeAnyLocTestCase(unittest.TestCase):
    def testEqual1(self):
        """Test default values for size, horizontal and vertical precision."""
        r1 = dns.rrset.from_text(
            "foo", 300, "IN", "LOC", "49 11 42.400 N 16 36 29.600 E 227.64m"
        )
        r2 = dns.rrset.from_text(
            "FOO",
            600,
            "in",
            "loc",
            "49 11 42.400 N 16 36 29.600 E 227.64m " "1.00m 10000.00m 10.00m",
        )
        self.assertEqual(r1, r2, '"{}" != "{}"'.format(r1, r2))

    def testEqual2(self):
        """Test default values for size, horizontal and vertical precision."""
        r1 = dns.rdtypes.ANY.LOC.LOC(
            1, 29, (49, 11, 42, 400, 1), (16, 36, 29, 600, 1), 22764.0
        )  # centimeters
        r2 = dns.rdtypes.ANY.LOC.LOC(
            1,
            29,
            (49, 11, 42, 400, 1),
            (16, 36, 29, 600, 1),
            22764.0,  # centimeters
            100.0,
            1000000.00,
            1000.0,
        )  # centimeters
        self.assertEqual(r1, r2, '"{}" != "{}"'.format(r1, r2))

    def testEqual3(self):
        """Test size, horizontal and vertical precision parsers: 100 cm == 1 m.

        Parsers in from_text() and __init__() have to produce equal results."""
        r1 = dns.rdtypes.ANY.LOC.LOC(
            1,
            29,
            (49, 11, 42, 400, 1),
            (16, 36, 29, 600, 1),
            22764.0,
            200.0,
            1000.00,
            200.0,
        )  # centimeters
        r2 = dns.rrset.from_text(
            "FOO",
            600,
            "in",
            "loc",
            "49 11 42.400 N 16 36 29.600 E 227.64m " "2.00m 10.00m 2.00m",
        )[0]
        self.assertEqual(r1, r2, '"{}" != "{}"'.format(r1, r2))

    def testEqual4(self):
        """Test size, horizontal and vertical precision parsers without unit.

        Parsers in from_text() and __init__() have produce equal result
        for values with and without trailing "m"."""
        r1 = dns.rdtypes.ANY.LOC.LOC(
            1,
            29,
            (49, 11, 42, 400, 1),
            (16, 36, 29, 600, 1),
            22764.0,
            200.0,
            1000.00,
            200.0,
        )  # centimeters
        r2 = dns.rrset.from_text(
            "FOO", 600, "in", "loc", "49 11 42.400 N 16 36 29.600 E 227.64 " "2 10 2"
        )[
            0
        ]  # meters without explicit unit
        self.assertEqual(r1, r2, '"{}" != "{}"'.format(r1, r2))


if __name__ == "__main__":
    unittest.main()



#!/usr/bin/env python3
#
# This is an example of sending DNS queries over HTTPS (DoH) with dnspython.
import httpx

import dns.message
import dns.query
import dns.rdatatype


def main():
    where = "https://dns.google/dns-query"
    qname = "example.com."
    with httpx.Client() as client:
        q = dns.message.make_query(qname, dns.rdatatype.A)
        r = dns.query.https(q, where, session=client)
        for answer in r.answer:
            print(answer)

        # ... do more lookups


if __name__ == "__main__":
    main()



#!/usr/bin/env python3

import dns.query
import dns.resolver
import dns.zone

soa_answer = dns.resolver.resolve("dnspython.org", "SOA")
master_answer = dns.resolver.resolve(soa_answer[0].mname, "A")

z = dns.zone.from_xfr(dns.query.xfr(master_answer[0].address, "dnspython.org"))
for n in sorted(z.nodes.keys()):
    print(z[n].to_text(n))



# Copyright (C) Dnspython Contributors, see LICENSE for text of ISC license

# Copyright (C) 2003-2007, 2009-2011 Nominum, Inc.
#
# Permission to use, copy, modify, and distribute this software and its
# documentation for any purpose with or without fee is hereby granted,
# provided that the above copyright notice and this permission notice
# appear in all copies.
#
# THE SOFTWARE IS PROVIDED "AS IS" AND NOMINUM DISCLAIMS ALL WARRANTIES
# WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
# MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL NOMINUM BE LIABLE FOR
# ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
# WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
# ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT
# OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.

import binascii
import itertools
import socket
import unittest

import dns.exception
import dns.inet
import dns.ipv4
import dns.ipv6

# for convenience
aton4 = dns.ipv4.inet_aton
ntoa4 = dns.ipv4.inet_ntoa
aton6 = dns.ipv6.inet_aton
ntoa6 = dns.ipv6.inet_ntoa

v4_bad_addrs = [
    "256.1.1.1",
    "1.1.1",
    "1.1.1.1.1",
    "+1.1.1.1",
    "1.1.1.1+",
    "1..2.3.4",
    ".1.2.3.4",
    "1.2.3.4.",
]

v4_canonicalize_addrs = [
    # (input, expected)
    ("127.0.0.1", "127.0.0.1"),
    (b"127.0.0.1", "127.0.0.1"),
]

v6_canonicalize_addrs = [
    # (input, expected)
    ("2001:503:83eb:0:0:0:0:30", "2001:503:83eb::30"),
    (b"2001:503:83eb:0:0:0:0:30", "2001:503:83eb::30"),
    ("2001:db8::1:1:1:1:1", "2001:db8:0:1:1:1:1:1"),
    ("2001:DB8::1:1:1:1:1", "2001:db8:0:1:1:1:1:1"),
]

bad_canonicalize_addrs = [
    "127.00.0.1",
    "hi there",
    "2001::db8::1:1:1:1:1",
    "fe80::1%lo0",
]


class NtoAAtoNTestCase(unittest.TestCase):
    def test_aton1(self):
        a = aton6("::")
        self.assertEqual(a, b"\x00" * 16)

    def test_aton2(self):
        a = aton6("::1")
        self.assertEqual(a, b"\x00" * 15 + b"\x01")

    def test_aton3(self):
        a = aton6("::10.0.0.1")
        self.assertEqual(a, b"\x00" * 12 + b"\x0a\x00\x00\x01")

    def test_aton4(self):
        a = aton6("abcd::dcba")
        self.assertEqual(a, b"\xab\xcd" + b"\x00" * 12 + b"\xdc\xba")

    def test_aton5(self):
        a = aton6("1:2:3:4:5:6:7:8")
        self.assertEqual(a, binascii.unhexlify(b"00010002000300040005000600070008"))

    def test_bad_aton1(self):
        def bad():
            aton6("abcd:dcba")

        self.assertRaises(dns.exception.SyntaxError, bad)

    def test_bad_aton2(self):
        def bad():
            aton6("abcd::dcba::1")

        self.assertRaises(dns.exception.SyntaxError, bad)

    def test_bad_aton3(self):
        def bad():
            aton6("1:2:3:4:5:6:7:8:9")

        self.assertRaises(dns.exception.SyntaxError, bad)

    def test_bad_aton4(self):
        def bad():
            aton4("001.002.003.004")

        self.assertRaises(dns.exception.SyntaxError, bad)

    def test_aton6(self):
        a = aton6("::")
        self.assertEqual(a, b"\x00" * 16)

    def test_aton7(self):
        a = aton6("::1")
        self.assertEqual(a, b"\x00" * 15 + b"\x01")

    def test_aton8(self):
        a = aton6("::10.0.0.1")
        self.assertEqual(a, b"\x00" * 12 + b"\x0a\x00\x00\x01")

    def test_aton9(self):
        a = aton6("abcd::dcba")
        self.assertEqual(a, b"\xab\xcd" + b"\x00" * 12 + b"\xdc\xba")

    def test_ntoa1(self):
        b = binascii.unhexlify(b"00010002000300040005000600070008")
        t = ntoa6(b)
        self.assertEqual(t, "1:2:3:4:5:6:7:8")

    def test_ntoa2(self):
        b = b"\x00" * 16
        t = ntoa6(b)
        self.assertEqual(t, "::")

    def test_ntoa3(self):
        b = b"\x00" * 15 + b"\x01"
        t = ntoa6(b)
        self.assertEqual(t, "::1")

    def test_ntoa4(self):
        b = b"\x80" + b"\x00" * 15
        t = ntoa6(b)
        self.assertEqual(t, "8000::")

    def test_ntoa5(self):
        b = b"\x01\xcd" + b"\x00" * 12 + b"\x03\xef"
        t = ntoa6(b)
        self.assertEqual(t, "1cd::3ef")

    def test_ntoa6(self):
        b = binascii.unhexlify(b"ffff00000000ffff000000000000ffff")
        t = ntoa6(b)
        self.assertEqual(t, "ffff:0:0:ffff::ffff")

    def test_ntoa7(self):
        b = binascii.unhexlify(b"00000000ffff000000000000ffffffff")
        t = ntoa6(b)
        self.assertEqual(t, "0:0:ffff::ffff:ffff")

    def test_ntoa8(self):
        b = binascii.unhexlify(b"ffff0000ffff00000000ffff00000000")
        t = ntoa6(b)
        self.assertEqual(t, "ffff:0:ffff::ffff:0:0")

    def test_ntoa9(self):
        b = binascii.unhexlify(b"0000000000000000000000000a000001")
        t = ntoa6(b)
        self.assertEqual(t, "::10.0.0.1")

    def test_ntoa10(self):
        b = binascii.unhexlify(b"0000000000000000000000010a000001")
        t = ntoa6(b)
        self.assertEqual(t, "::1:a00:1")

    def test_ntoa11(self):
        b = binascii.unhexlify(b"00000000000000000000ffff0a000001")
        t = ntoa6(b)
        self.assertEqual(t, "::ffff:10.0.0.1")

    def test_ntoa12(self):
        b = binascii.unhexlify(b"000000000000000000000000ffffffff")
        t = ntoa6(b)
        self.assertEqual(t, "::255.255.255.255")

    def test_ntoa13(self):
        b = binascii.unhexlify(b"00000000000000000000ffffffffffff")
        t = ntoa6(b)
        self.assertEqual(t, "::ffff:255.255.255.255")

    def test_ntoa14(self):
        b = binascii.unhexlify(b"0000000000000000000000000001ffff")
        t = ntoa6(b)
        self.assertEqual(t, "::0.1.255.255")

    def test_ntoa15(self):
        # This exercises the current_len > best_len branch in the <= case.
        b = binascii.unhexlify(b"0000ffff00000000ffff00000000ffff")
        t = ntoa6(b)
        self.assertEqual(t, "0:ffff::ffff:0:0:ffff")

    def test_bad_ntoa1(self):
        def bad():
            ntoa6(b"")

        self.assertRaises(ValueError, bad)

    def test_bad_ntoa2(self):
        def bad():
            ntoa6(b"\x00" * 17)

        self.assertRaises(ValueError, bad)

    def test_bad_ntoa3(self):
        def bad():
            ntoa4(b"\x00" * 5)

        # Ideally we'd have been consistent and raised ValueError as
        # we do for IPv6, but oh well!
        self.assertRaises(dns.exception.SyntaxError, bad)

    def test_good_v4_aton(self):
        pairs = [
            ("1.2.3.4", b"\x01\x02\x03\x04"),
            ("255.255.255.255", b"\xff\xff\xff\xff"),
            ("0.0.0.0", b"\x00\x00\x00\x00"),
        ]
        for t, b in pairs:
            b1 = aton4(t)
            t1 = ntoa4(b1)
            self.assertEqual(b1, b)
            self.assertEqual(t1, t)

    def test_bad_v4_aton(self):
        def make_bad(a):
            def bad():
                return aton4(a)

            return bad

        for addr in v4_bad_addrs:
            self.assertRaises(dns.exception.SyntaxError, make_bad(addr))

    def test_bad_v6_aton(self):
        addrs = ["+::0", "0::0::", "::0::", "1:2:3:4:5:6:7:8:9", ":::::::"]
        embedded = ["::" + x for x in v4_bad_addrs]
        addrs.extend(embedded)

        def make_bad(a):
            def bad():
                x = aton6(a)

            return bad

        for addr in addrs:
            self.assertRaises(dns.exception.SyntaxError, make_bad(addr))

    def test_rfc5952_section_4_2_2(self):
        addr = "2001:db8:0:1:1:1:1:1"
        b1 = aton6(addr)
        t1 = ntoa6(b1)
        self.assertEqual(t1, addr)

    def test_is_mapped(self):
        t1 = "2001:db8:0:1:1:1:1:1"
        t2 = "::ffff:127.0.0.1"
        t3 = "1::ffff:127.0.0.1"
        self.assertFalse(dns.ipv6.is_mapped(aton6(t1)))
        self.assertTrue(dns.ipv6.is_mapped(aton6(t2)))
        self.assertFalse(dns.ipv6.is_mapped(aton6(t3)))

    def test_is_multicast(self):
        t1 = "223.0.0.1"
        t2 = "240.0.0.1"
        t3 = "224.0.0.1"
        t4 = "239.0.0.1"
        t5 = "fe00::1"
        t6 = "ff00::1"
        self.assertFalse(dns.inet.is_multicast(t1))
        self.assertFalse(dns.inet.is_multicast(t2))
        self.assertTrue(dns.inet.is_multicast(t3))
        self.assertTrue(dns.inet.is_multicast(t4))
        self.assertFalse(dns.inet.is_multicast(t5))
        self.assertTrue(dns.inet.is_multicast(t6))

    def test_is_multicast_bad_input(self):
        def bad():
            dns.inet.is_multicast("hello world")

        self.assertRaises(ValueError, bad)

    def test_ignore_scope(self):
        t1 = "fe80::1%lo0"
        t2 = "fe80::1"
        self.assertEqual(aton6(t1, True), aton6(t2))

    def test_do_not_ignore_scope(self):
        def bad():
            t1 = "fe80::1%lo0"
            aton6(t1)

        self.assertRaises(dns.exception.SyntaxError, bad)

    def test_multiple_scopes_bad(self):
        def bad():
            t1 = "fe80::1%lo0%lo1"
            aton6(t1, True)

        self.assertRaises(dns.exception.SyntaxError, bad)

    def test_ptontop(self):
        for af, a in [
            (socket.AF_INET, "1.2.3.4"),
            (socket.AF_INET6, "2001:db8:0:1:1:1:1:1"),
        ]:
            self.assertEqual(dns.inet.inet_ntop(af, dns.inet.inet_pton(af, a)), a)

    def test_isaddress(self):
        for t, e in [
            ("1.2.3.4", True),
            ("2001:db8:0:1:1:1:1:1", True),
            ("hello world", False),
            ("http://www.dnspython.org", False),
            ("1.2.3.4a", False),
            ("2001:db8:0:1:1:1:1:q1", False),
        ]:
            self.assertEqual(dns.inet.is_address(t), e)

    def test_low_level_address_tuple(self):
        t = dns.inet.low_level_address_tuple(("1.2.3.4", 53))
        self.assertEqual(t, ("1.2.3.4", 53))
        t = dns.inet.low_level_address_tuple(("2600::1", 53))
        self.assertEqual(t, ("2600::1", 53, 0, 0))
        t = dns.inet.low_level_address_tuple(("1.2.3.4", 53), socket.AF_INET)
        self.assertEqual(t, ("1.2.3.4", 53))
        t = dns.inet.low_level_address_tuple(("2600::1", 53), socket.AF_INET6)
        self.assertEqual(t, ("2600::1", 53, 0, 0))
        t = dns.inet.low_level_address_tuple(("fd80::1%2", 53), socket.AF_INET6)
        self.assertEqual(t, ("fd80::1", 53, 0, 2))
        try:
            # This can fail on windows for python < 3.8, so we tolerate
            # the failure and only test if we have something we can work
            # with.
            info = socket.if_nameindex()
        except Exception:
            info = []
        if info:
            # find first thing on list that is not zero (should be first thing!
            pair = None
            for p in info:
                if p[0] != 0:
                    pair = p
                    break
            if pair:
                address = "fd80::1%" + pair[1]
                t = dns.inet.low_level_address_tuple((address, 53), socket.AF_INET6)
                self.assertEqual(t, ("fd80::1", 53, 0, pair[0]))

        def bad():
            bogus = socket.AF_INET + socket.AF_INET6 + 1
            t = dns.inet.low_level_address_tuple(("2600::1", 53), bogus)

        self.assertRaises(NotImplementedError, bad)

    def test_bogus_family(self):
        self.assertRaises(
            NotImplementedError, lambda: dns.inet.inet_pton(12345, "bogus")
        )
        self.assertRaises(
            NotImplementedError, lambda: dns.inet.inet_ntop(12345, b"bogus")
        )

    def test_ipv4_canonicalize(self):
        for address, expected in v4_canonicalize_addrs:
            self.assertEqual(dns.ipv4.canonicalize(address), expected)
        for bad_address in bad_canonicalize_addrs:
            self.assertRaises(
                dns.exception.SyntaxError, lambda: dns.ipv4.canonicalize(bad_address)
            )

    def test_ipv6_canonicalize(self):
        for address, expected in v6_canonicalize_addrs:
            self.assertEqual(dns.ipv6.canonicalize(address), expected)
        for bad_address in bad_canonicalize_addrs:
            self.assertRaises(
                dns.exception.SyntaxError, lambda: dns.ipv6.canonicalize(bad_address)
            )

    def test_inet_canonicalize(self):
        for address, expected in itertools.chain(
            v4_canonicalize_addrs, v6_canonicalize_addrs
        ):
            self.assertEqual(dns.inet.canonicalize(address), expected)
        for bad_address in bad_canonicalize_addrs:
            self.assertRaises(ValueError, lambda: dns.inet.canonicalize(bad_address))


if __name__ == "__main__":
    unittest.main()
